/* src/admin_user.c - commandes admins pour gerer les users
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/time.h>
#include "main.h"
#include <dirent.h>
#ifdef USEBSD
#include <netinet/in.h>
#endif
#include <arpa/inet.h>

#define MAXMATCHES 40

extern int dataQ;
extern int dataS;
extern struct badchans *badchead;
extern struct badnicks *badnhead;
extern struct ignore *ignorehead;
extern struct Link *mainhub;
extern struct cs_commands cs_cmds[];
extern void uptime(aNick *nick, aChan *chaninfo, int parc, char **parv);

static int b2n (int b)
{
	int i = 0;
	char bbbase64[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789[]";

	for (; i<64; i++) if (bbbase64[i] == b) return i;

	return i;
}

static char *GetIP(char *base64)
{
	int i = 5, poids = 1, addr = 0;
	struct in_addr addr2;
	static char Ip[16];

	for(;i >= 0; i--)
	{
		addr += poids * b2n(base64[i]);
		poids = poids * 64;
	}

	addr2.s_addr = htonl(addr);
	strncpy(Ip, inet_ntoa(addr2), 16);
	return Ip;
}

void cs_whois(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *user = parv[1], chans[450] = {0}, b[120] = {0}, *p;
	int auth = (nick->nick == user || IsAnAdmin(nick->user)), size = 0, first = 0;
	aNick *tmp;
	aJoin *chanl;
	anAccess *acces;
	anUser *ux;

	if(!(tmp = getnickbynick(user)))
	{
		csntc(nick, NOTCONNECTED, user);
		return;
	}

	csntc(nick, "Infos sur \2%s", tmp->nick);
	csntc(nick, " Nom: %s", tmp->name);
#ifdef HAVE_CRYPTHOST
	csntc(nick, " Mask: %s\2%s\2!\002%s\2@\002%s", GetPrefix(tmp), tmp->nick, tmp->ident, tmp->crypt);
	if(auth) csntc(nick, " Realhost: \002%s\2", tmp->host);
#else
	csntc(nick, " Mask: %s\2%s\2!\002%s\2@\002%s", GetPrefix(tmp), tmp->nick, tmp->ident, IsAnAdmin(nick->user) ? tmp->host : "*");
#endif
	if(auth) csntc(nick, " IP: \002%s", GetIP(tmp->base64));
	if((tmp->flag & N_OPER)) csntc(nick, " %s est \2IRCop.", tmp->nick);
	p = GetModes(tmp);
	if(auth) csntc(nick, " UserModes: \2%s\2", (p[0] != '\0') ? p : "Aucun");
	if(auth)
	{
		for(chanl = tmp->joinhead;chanl;chanl=chanl->next)
		{
			snprintf(b, 120, "%s%s%s ", chanl->is_voice ? "+" : "", chanl->is_op ? "@" : "", chanl->channel);
			size += strlen(b);
			if(size >= 450) /* le dernier fait depasser le buffer */
			{
				strcat(chans, " ...\0");
				csntc(nick, "%s%s", !first ? " Il est sur: " : "", chans);
				strcpy(chans, b); /* ajoute le suivant pour la prochaine*/
				first = 1;
				size = strlen(b);
			}
			else strcat(chans, b);
		}
		if(chans[0] == '\0') strcpy(chans, "Aucun salon");
		strcat(chans, "\0");

		csntc(nick, "%s%s", !first ? " Il est sur: " : "", chans);
	}
	if(tmp->user)
	{
		csntc(nick, " %s est logu sous l'username \2%s\2%s", user, tmp->user->nick,\
		(tmp->user->level > 1) ? " (\2Admin\2)" : "");
		if(auth)
		{
			chans[0] = '\0';
			size = first = 0;

			for(acces = tmp->user->accesshead;acces;acces=acces->next)
			{
				if((acces->flag & A_WAITACCESS)) continue;
				snprintf(b, 120, "%s ", acces->chan);
				size += strlen(b);
				if(size >= 450)
				{
					strcat(chans, " ...\0");
					csntc(nick, "%s%s", !first ? " Il a accs : " : "", chans);
					strcpy(chans, b); /* ajoute le suivant pour la prochaine*/
					first = 1;
					size = strlen(b);
				}
				else strcat(chans, b);
			}
			strcat(chans, "\0");

			if(chans[0] != '\0') csntc(nick, "%s%s", !first ? " Il a accs : " : "", chans);
			else csntc(nick, " Il n'a aucun accs.");
		}
	}
	else
	{
		if((ux = getuserinfo(user))) csntc(nick, " %s est un username enregistr. %s", \
		ux->nick, IsAnAdmin(ux) ? " (\2Admin\2)" : "");
	}

	csntc(nick, " Connect depuis %s sur \2%s", duration(time(NULL) - tmp->ttmco), tmp->serveur?tmp->serveur->serv:bot.server);
	return;
}

void admin_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *user = parv[2];
	const char *arg = parv[1];
	anUser *u = NULL;
	anAccess *a, *a2;
	aNick *n;

	if(!strcasecmp(arg, "info"))
	{
		int mail, seen, date, auth, suspend, count, oubli, first, nopurge, cur = 0;
		time_t ttm = time(NULL);
		char tmp[75];

		if(*parv[2] == '-') user = "*";

		mail=getoption("-mail",parv,parc,2,0);
		seen=getoption("-seen",parv,parc,2,1);
		date=getoption("-date",parv,parc,2,1);
		count=getoption("-count",parv,parc,2,1);
		auth=getoption("-auth",parv,parc,2,-1);
		suspend=getoption("-suspend",parv,parc,2,-1);
		oubli=getoption("-oubli",parv,parc,2,-1);
		first=getoption("-first",parv,parc,2,-1);
		nopurge=getoption("-nopurge",parv,parc,2,-1);

		if(strlen(user) > NICKLEN) return;

		for(u=userhead;u;u=u->next)
		{
			char chans[451] = {0}, chansw[500] = {0}, b[120];
			int size = 0, firstt = 0, sizew = 0, firstw = 0;
			if((!mail || !match(parv[mail],u->mail))
			&& (!seen || (ttm - u->lastseend) <= seen)
			&& (!date || ttm >= u->lastseend)
			&& (!auth || (auth && u->n) || (!auth && !u->n))
			&& (!suspend || IsSuspend(u))
			&& (!oubli || UOubli(u))
			&& (!first || UFirst(u))
			&& (!nopurge || UNopurge(u))
			&& (!match(user,u->nick)))
			{
				cur++;
				if(!count || cur <= count)
				{
					csntc(nick, "Infos sur l'username \2%s\2 :", u->nick);
					csntc(nick, " Mail: %s", u->mail);

					if(UFirst(u)) csntc(nick, " En cours de validation, enregistrement il y a %s", duration(time(NULL) - u->lastseend));
					else csntc(nick, " Logu pour la dernire fois il y a %s", duration(time(NULL) - u->lastseend));
					if(UOubli(u)) csntc(nick, " A utilis la commande oubli");
					if(IsAnAdmin(u)) csntc(nick, " \2Accs Administrateur.\2 (\002%d\2)", u->level);

					for(a = u->accesshead;a;a=a->next)
					{
						snprintf(b, 120, "%s (%d) ", a->chan, a->level);
						if((a->flag & A_WAITACCESS))
						{
							sizew += strlen(b);
							if(sizew >= 450) /* le dernier fait depasser le buffer */
							{
								strcat(chansw, " ...\0");
								csntc(nick, "%s%s", !firstw ? " Il a des propositions sur: " : "", chansw);
								strcpy(chansw, b); /* ajoute le suivant pour la prochaine*/
								firstw = 1;
								sizew = strlen(b);
							}
							else strcat(chansw, b);
						}
						else
						{
							size += strlen(b);
							if(size >= 450) /* le dernier fait depasser le buffer */
							{
								strcat(chans, " ...\0");
								csntc(nick, "%s%s", !firstt ? " Il a accs : " : "", chans);
								strcpy(chans, b); /* ajoute le suivant pour la prochaine*/
								firstt = 1;
								size = strlen(b);
							}
							else strcat(chans, b);
						}
					}
					strcat(chans, "\0");
					strcat(chansw, "\0");
					if(chans[0] != '\0') csntc(nick, "%s%s", !first ? " Il a accs : " : "", chans);
					else if(!firstt) csntc(nick, " Il n'a aucun accs.");
					if(chansw[0] != '\0') csntc(nick, "%s%s", !firstw ? " Il a des propositions sur: " : "", chansw);
					else if(!firstw) csntc(nick, " Il n'a aucune proposition d'access.");

					if(!(u->flag & U_POKACCESS)) csntc(nick, " Il %s systmatiquement les propositions d'access.", (u->flag & U_PACCEPT) ? "accepte" : "refuse");
					if((n = u->n)) csntc(nick, " %s%s est actuellement logu sous cet Username (%s@%s)",\
					GetPrefix(n), n->nick, n->ident, n->host);
					else csntc(nick, " Personne n'est actuellement logu sous cet username");
					/*if(u->vote) csntc(nick, "Il a vot pour %s", vote[(u->vote-1)].prop);*/

					if(u->suspend) show_ususpend(nick, u);
					csntc(nick, " Options: %s", GetUserOptions(u));
					csntc(nick, " Last Login depuis %s", u->lastlogin ? u->lastlogin : "<Non Disponible>");

					if(!count || count>1) csntc(nick, "-");
				}
			}
		}

		if(cur)
		{
			if(count && count < cur) sprintf(tmp,"(%d listes)", count);
			else tmp[0]='\0';
			csntc(nick, "Un Total de %d entre%s trouve%s %s", cur, cur > 1 ? "s" : "", cur > 1 ? "s": "", tmp);
		}
		else csntc(nick, "Aucun Username correspondand  \002%s\2 trouv.", user);
		return;
	}

	if(!strcasecmp(arg, "match"))
	{
		char *p = parv[2];
		struct timeval tv;
		long ticks;
		struct Link *link= NULL;

		gettimeofday(&tv, NULL);
		ticks = tv.tv_usec;

		if(*p == '+' || *p == '-')
		{
			int w = 1, host = 0, nicki = 0, ident = 0, real = 0, join = 0, par = 2;
			int mode = 0, ac = 0, count = 0, crypt = 0, serv = 0, nolimit = 0;

			while(*p)
			{
				switch(*p)
				{
					case '+':
							w = 1;
							break;
					case '-':
							w = 0;
							break;
					case 'h':
							if((host = ++par) <= parc)
							{
								if(!w) host = -1 * host;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 'n':
							if((nicki = ++par) <= parc)
							{
								if(!w) nicki = -1 * nicki;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 'u':
							if((ident = ++par) <= parc)
							{
								if(!w) ident = -1 * ident;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 'g':
							if((real = ++par) <= parc)
							{
								if(!w) real = -1 * real;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 'c':
							if((join = ++par) <= parc)
							{
								if(!w) join = -1 * join;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 'm':
							if((mode = ++par) <= parc)
							{
								if(!w) mode = -1 * mode;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 'a':
							if((ac = ++par) <= parc)
							{
								if(!w) ac = -1 * ac;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
					case 's':
							if((serv = ++par) <= parc)
							{
								for(link=mainhub;link && strcasecmp(link->serv, parv[serv]);link=link->next);
								if(!link)
								{
									csntc(nick, "Serveur %s inconnu.", parv[serv]);
									return;
								}
								if(!w) serv = -1 * serv;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
#ifdef HAVE_CRYPTHOST
					case 'C':
							if((crypt = ++par) <= parc)
							{
								if(!w) crypt = -1 * crypt;
								break;
							}
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
#endif
					default:
							csntc(nick, "Syntaxe %s MATCH [+|-]nughCcsam [args..]", RealCmd("user"));
							return;
				}
				p++;
			}
			if(!strcasecmp("-all", parv[parc])) nolimit = 1;
			for(n=nickhead;n;n=n->next)
			{
				if(
					(!nicki || (nicki > 0 && !match(parv[nicki], n->nick)) || (nicki < 0 && match(parv[-1 * nicki], n->nick)))
					&& (!ident || (ident > 0 && !match(parv[ident], n->ident)) || (ident < 0 && match(parv[-1 * ident], n->ident)))
					&& (!real || (real > 0 && !match(parv[real], n->name)) || (real < 0 && match(parv[-1 * real], n->name)))
					&& (!serv || (serv > 0 && link == n->serveur) || (serv < 0 && link != n->serveur))
					&& (!host || (host > 0 && !match(parv[host], n->host)) || (host < 0 && match(parv[-1 * host], n->host)))
					&& (!mode || (mode > 0 && !match(parv[mode], GetModes(n))) || (mode < 0 && match(parv[-1 * mode], GetModes(n))))
					&& (!ac || (ac > 0 && n->user && !match(parv[ac], n->user->nick)) || (ac < 0 && n->user && match(parv[-1 * ac], n->user->nick)))
#ifdef HAVE_CRYPTHOST
					&& (!crypt || (crypt > 0 && !match(parv[crypt], n->crypt)) || (crypt < 0 && match(parv[-1 * crypt], n->crypt)))
#endif
					&& (!join || (join > 0 && getjoininfo(n, parv[join])) || (join < 0 && !getjoininfo(n, parv[-1 * join])))
					&& (++count <= MAXMATCHES || nolimit))
						csntc(nick, "Matching Nick: [%d] \2%s%s\2 -> %s@%s", count, GetPrefix(n), n->nick, n->ident, n->host);
			}

			gettimeofday(&tv, NULL);
			if(count > MAXMATCHES && !nolimit) csntc(nick, "[%d] Trop de nicks trouvs, restreignez votre recherche ou forcez avec '-all' en DERNIER argument.", MAXMATCHES);
			csntc(nick, "Fin de la Recherche: %d matche%s trouve%s. (Effectue en %6.3f ms)"\
			, count, (count > 1) ? "s" : "", (count > 1) ? "s" : "", (float) (tv.tv_usec - ticks)/1000);
		}
		else
		{
			char host[USERLEN + HOSTLEN + 3], b[100];
			int i = 0, d = 0;

			if(parc > 2) d = atoi(parv[3]);

			csntc(nick, "Listing Nicks...");

			for(n = nickhead;n;n=n->next)
			{
				snprintf(host, (USERLEN + HOSTLEN + 2), "%s@%s", n->ident, n->host);
				if(!match(user, host)) if(!d || (i++ < d)) csntc(nick, "Matching Nick: \2%s%s\2 -> %s", GetPrefix(n), n->nick, host);
			}

			if(d && (i > d)) snprintf(b, 100, " (%d entres listes.)", d);
			else strcpy(b, " ");

			gettimeofday(&tv, NULL);
			csntc(nick, "Fin de la Recherche sur\2 %s\2 : %d matche%s trouve%s%s. (Effectue en %6.3f ms)"\
			, user, i, (i > 1) ? "s" : "", (i > 1) ? "s" : "", b,(float) (tv.tv_usec - ticks)/1000);
		}
		return;
	}

	if(!strcasecmp(arg, "suspend"))
	{
		if(!strcasecmp(nick->user->nick, user))
		{
			csntc(nick, "Vous ne devriez pas vous suspendre.. (Utilisez la commande SUSPENDME)");
			return;
		}

		if(!(u = ParseNickOrUser(nick, user))) return;

		if(IsAnAdmin(u) && u->level >= nick->user->level)
		{
			csntc(nick, "Vous ne pouvez pas suspendre un Admin de niveau suprieur ou gal.");
			return;
		}

		if(parc < 3 && !IsSuspend(u))
		{
			csntc(nick, "Veuillez prciser une raison pour suspendre cet UserName.");
			return;
		}
		if(!IsSuspend(u))
		{
			do_user_suspend(u, parv2msg(parc, parv, 3, 250), nick->user->nick, time(NULL));
			show_ususpend(nick, u);

			if((n = u->n))
			{
				n->user->n = NULL;
				n->user = NULL;
				csntc(n, "Votre Username vient d'tre suspendu par l'Admin %s.", nick->nick);
				cs_account(n, NULL);
			}
		}
		else
		{
			int nolog = getoption("-nolog", parv, parc, 2, -1);

			if(nolog)
			{
				free(u->suspend);
				u->suspend = NULL;
			}
			else u->suspend->debut = -1 * u->suspend->debut;

			csntc(nick, "L'Username \2%s\2 n'est plus suspendu.", u->nick);

		}
		write_anuser(u);
		return;
	}

	if(!strcasecmp(arg, "mail"))
	{
		if(parc < 3 || !IsValidMail(parv[3]))
		{
			csntc(nick, "Veuillez prciser un Mail valide.");
			return;
		}
		if(!(u = ParseNickOrUser(nick, user))) return;

		strncpy(u->mail, parv[3], 100);
		u->mail[99] = 0;
		csntc(nick, "Le mail de %s a t chang en \2%s", u->nick, parv[3]);
		write_anuser(u);
		return;
	}

	if(!strcasecmp(arg, "newpass"))
	{
		char p[15];
		if(parc < 3 || strlen(parv[3]) < 6)
		{
			csntc(nick, "Veuillez entrer un pass (Valide).");
			return;
		}
		if(!(u = ParseNickOrUser(nick, user))) return;

		if(IsAnAdmin(u) && u->level >= nick->user->level && u != nick->user)
		{
			csntc(nick, "Vous ne pouvez pas changer le pass d'un Admin de niveau suprieur ou gal.");
			return;
		}

		strncpy(p,parv[3],15);
		strncpy(u->passwd, mkpass(p, "lala"), 20);
		csntc(nick, "Le mot de pass de \2%s\2 a bien t chang.", user);
		write_anuser(u);
		return;
	}

	if(!strcasecmp(arg, "newnick"))
	{
		char *newusr = NULL;
		anUser *utmp;
		int found = 0;

		if(parc < 3 || strlen(parv[3]) > NICKLEN || !IsValidNick(parv[3]))
		{
			csntc(nick, "Veuillez entrer un username (Valide).");
			return;
		}

		newusr = parv[3];
		for(utmp=userhead;utmp;utmp=utmp->next)
		{
			if(!strcasecmp(utmp->nick, newusr))
			{
				csntc(nick, "%s est un username dj utilis", newusr);
				return;
			}
			if(!strcasecmp(utmp->nick, user))
			{
				found = 1;
				u = utmp;
			}
		}
		if(!found)
		{
			csntc(nick, NOSUCHUSER, user);
			return;
		}

		if(IsAnAdmin(u) && u->level >= nick->user->level && strcasecmp(u->nick,nick->user->nick))
		{
			csntc(nick, "Vous ne pouvez pas changer l'username d'un Admin de niveau suprieur ou gal.");
			return;
		}

		File_UserDel(u->nick);
		strncpy(u->nick, newusr, NICKLEN);
		u->nick[NICKLEN] = 0;
		if(u->n) cs_account(u->n, newusr);
		csntc(nick, "L'username de %s a t chang en %s.", user, newusr);
		write_anuser(u);
		return;
	}

	if(!strcasecmp(arg, "del"))
	{
		if(!strcasecmp(nick->user->nick, user))
		{
			csntc(nick, "Vous ne devriez pas vous deluser.. (Utilisez la commande DROP)");
			return;
		}
		if(!(u = ParseNickOrUser(nick, user))) return;

		if(IsAnAdmin(u) && u->level >= nick->user->level)
		{
			csntc(nick, "Vous ne pouvez supprimer un Username d'Admin de niveau suprieur ou gal.");
			return;
		}
		if((n = u->n))
		{
			char raison[200];
			if(parc > 2) parv2msgn(parc, parv, 3, raison, 200);
			else strcpy(raison, "");

			cs_account(n, NULL);
			csntc(n, "Votre Username vient d'tre supprim par l'Admin %s %s", nick->nick, raison);
			n->user->n = NULL;
			n->user = NULL;
		}

		csntc(nick, "L'username %s a bien t supprim.", u->nick);

		for(a=u->accesshead;a;a=a2)
		{
			a2=a->next;
			if(a->level == 500)
			{
				csntc(nick, "Le salon %s a t dsenregistr (%s en tait l'owner)", a->chan, u->nick);
				cspart(a->chan, "Salon dsenregistr par %s -\2 Suppression de l'username de l'owner", nick->nick);
				cswallops("Le salon %s a t dsenregistr par %s -\2 Suppression de l'username de l'owner",a->chan, nick->nick);
				putlog(LOG_CHANS, "CHAN DEL %s par %s@%s (%s)\n", a->chan, nick->nick, nick->user->nick, "USER DEL de l'owner");

				del_chan(a->chan);
			}
		}
		del_regnick(u);
		write_userlist();
		return;
	}

	if(!strcasecmp(arg, "stats"))
	{
		int mem,cmds,traffic,all;
		mem=getoption("-mem",parv,parc,2,-1);
		cmds=getoption("-cmds",parv,parc,2,0);
		traffic=getoption("-traffic",parv,parc,2,-1);
		all=getoption("-all",parv,parc,2,-1);

		if(mem || all)
		{
			int mem = 0, usr = 0, chs = 0, usra = 0, ni = 0, maxbans, maxaccess, ma, mb;
			aBan *ban;
			aJoin *j;
			struct badnicks *bn;
			struct badchans *bc;
			struct ignore *ig;
			aLink *lp;

			maxbans = getoption("-maxbans", parv, parc, 2, 1);
			maxaccess = getoption("-maxaccess", parv, parc, 2, 1);
			maxbans = maxbans ? maxbans : 20;
			maxaccess = maxaccess ? maxaccess : 20;

			uptime(nick, chaninfo, parc, parv);

			for(n=nickhead;n;n = n->next)
			{
				ni++;
				for(j=n->joinhead;j;j = j->next) mem += sizeof(aJoin);
			}
			mem += ni * sizeof(aNick);
			for(u=userhead;u;u = u->next)
			{
				usr++;
				if(u->suspend) mem += sizeof(struct suspendinfo);
				if(u->n) usra++;
			}
			mem += usr * sizeof(anUser);
			for(chaninfo = chanhead;chaninfo;chaninfo = chaninfo->next)
			{
				chs++;
				if(chaninfo->suspend) mem += sizeof(struct suspendinfo);
				mb = ma = 1;
				for(ban=chaninfo->banhead;ban;ban = ban->next, mb++);
				mem += mb * sizeof(aBan);
				if(mb >= maxbans) csntc(nick, "Warning Bans Limite: \2\00304%d\2\3 Bans sur %s", mb, chaninfo->chan);

				for(lp = chaninfo->access;lp;lp=lp->next, ma++) if(lp->value.a->info) mem += strlen(lp->value.a->info);
				mem += ma * sizeof(anAccess);
				if(ma >= maxaccess) csntc(nick, "Warning Acces Limite: \2\00304%d\2\3 Accs sur %s", ma, chaninfo->chan);
			}
			mem += chs * sizeof(aChan);
			for(bn=badnhead;bn;bn = bn->next) mem += sizeof(struct badnicks);
			for(bc=badchead;bc;bc = bc->next) mem += sizeof(struct badchans);
			for(ig=ignorehead;ig;ig=ig->next) mem += sizeof(struct ignore);
			mem += sizeof(struct robot) + sizeof(struct bots) + sizeof(struct Link);

			csntc(nick, "Memoire Utilise pour les Users/Chans: %10.3f KB", mem / 1024.0);
			csntc(nick, "Il y a %d Usernames enregistrs et %d Chans.", usr, chs);
			csntc(nick, "%d Chatteurs connects, dont %4.1f %% actuellement logus.", ni, (usra / (float) ni) * 100);
		}
		if(cmds || all)
		{
			int x = 0, adm = 0, usr = 0;

			for(;cs_cmds[x].cmd[0]!='\0';x++)
			{
				if(cs_cmds[x].level > 1 && cs_cmds[x].level < 5) adm += cs_cmds[x].used;
				else usr += cs_cmds[x].used;
				if(cmds && !match(parv[cmds],cs_cmds[x].cmd) && cs_cmds[x].cmd[0]!='\1') csntc(nick, "- %s (\002%d\2)", cs_cmds[x].cmd, cs_cmds[x].used);
			}
			csntc(nick, "Un total de %d commandes utilises, dont %d Admins (Ratio: %5.2f%)", (adm + usr), adm, (adm / (float) (usr + adm)) * 100);
		}
		if(traffic || all)
		{
			float up, down;
			int mbu = 0,mbd = 0;
			up = dataS/1024.0;
			down = dataQ/1024.0;
			if(up >= 1024)
			{
				up = up/1024.0;
				mbu = 1;
			}
			if(down >= 1024)
			{
				down = down/1024.0;
				mbd = 1;
			}
			csntc(nick, "Traffic: Up: \2%10.3f %s\2 Down: \2%10.3f %s\2", up, mbu ? "MB":"KB", down, mbd ? "MB":"KB");
		}
		if(!all && !mem && !traffic && !cmds) csntc(nick, "Option de stats non valide");
		return;
	}

	if(!strcasecmp(arg,"list"))
	{
		DIR *dp;
		struct dirent *ep;
		anUser *t;
		aChan *chan;
		char p[200], path[120];
		int got = 0, stat = 0, purge = getoption("-purge",parv,parc,2,-1);
		struct timeval tv;
		long ticks;

		gettimeofday(&tv, NULL);
		ticks = tv.tv_usec;

		if(!strcasecmp(parv[2],"user"))
		{
			if((dp=opendir("database/usr/")))
			{
				while((ep = readdir(dp)))
				{
					if(ep->d_name[0]!='.')
					{
						got=0;
						for(t=userhead;t;t=t->next)
						{
							conv_char(t->nick, p);
							if(!strcasecmp(p, ep->d_name)) { got=1; break; }
						}
						if(!got)
						{
							stat++;
							if(purge)
							{
								snprintf(path,120,"database/usr/%s", ep->d_name);
								remove(path);
							}
							csntc(nick, "Fichier User orphelin:\2 %s", ep->d_name);

						}
					}
				}
				closedir(dp);
			}
			else csntc(nick,"Impossible d'acceder  la database.");
		}
		if(!strcasecmp(parv[2],"chan"))
		{
			if((dp=opendir("database/chans/")))
			{
				while((ep = readdir(dp)))
				{
					if(ep->d_name[0]!='.')
					{
						got=0;
						for(chan=chanhead;chan;chan=chan->next)
						{
							conv_char(chan->chan, p);
							if(!strcasecmp(p, ep->d_name)) { got=1; break; }
						}
						if(!got)
						{
							stat++;
							if(purge)
							{
								snprintf(path,120,"database/chans/%s", ep->d_name);
								remove(path);
							}
							csntc(nick, "Fichier Chan orphelin: \2%s", ep->d_name);

						}
					}
				}
				closedir(dp);
			}
			else csntc(nick,"Impossible d'accder  la database.");
		}
		gettimeofday(&tv, NULL);
		csntc(nick, "%d fichiers %s en %6.3f ms.", stat, purge ? "purgs" : "lists", (float) (tv.tv_usec - ticks) / 1000);
		return;
	}
	if(!strcasecmp(arg, "nopurge"))
	{
		if(!(u = ParseNickOrUser(nick, user))) return;

		if(parc < 3)
		{
			if(UNopurge(u)) DelUNopurge(u);
			else SetUNopurge(u);
		}
		else
		{
			if(!strcasecmp(parv[1], "ON")) SetUNopurge(u);
			else if(!strcasecmp(parv[1], "OFF")) DelUNopurge(u);
			else
			{
				csntc(nick, "La valeur doit tre ON ou OFF");
				return;
			}
		}
		csntc(nick, "Le NOPURGE de %s est %sactif", u->nick, UNopurge(u) ? "" : "in");
		write_anuser(u);
		return;
	}
	if(!strcasecmp(arg, "checkpass"))
	{
		if(!(u = ParseNickOrUser(nick, user))) return;

		if(parc < 3)
		{
			csntc(nick, "Syntaxe: %s CHECKPASS <username> <pass>", RealCmd("user"));
			return;
		}

		if(!checkpass(parv[3], u))
		{
			csntc(nick, "Le mot de passe n'est \2PAS\2 correct !", u->nick);
			return;
		}

		csntc(nick, "Le mot de passe est correct !", u->nick);
		return;
	}

	csntc(nick, "%s n'est pas une option valide.", arg);
	return;
}

void do_user_suspend(anUser *user, const char *raison, const char *nick, long int debut)
{
	if(!user->suspend) user->suspend = calloc(1, sizeof(struct suspendinfo));
	strncpy(user->suspend->raison, raison, 250);
	strncpy(user->suspend->from, nick, NICKLEN);
	user->suspend->debut = debut;
	return;
}

void show_ususpend(aNick *nick, anUser *user)
{
	if(!user->suspend) return; /* on aurait jamais du l'envoyer l*/
	csntc(nick, "L'UserName %s \2%s\2 suspendu par %s le %s", user->nick, IsSuspend(user) ? "EST" : "tait", \
	user->suspend->from, get_time((user->suspend->debut > 0) ? user->suspend->debut : -1 * user->suspend->debut));
	csntc(nick, "Raison: %s", user->suspend->raison);
	return;
}
