/* src/ban.c - Commandes en rapport avec le ban
 * Copyright (C) 2002-2003 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <main.h>

CVSINFO("$Id: ban.c,v 1.14 2003/05/03 16:51:46 romexzf Exp $");

/*
 * ban_cmd - Progs 11 september 2002 (c l'anniversaire des attentats qui m'y a f pens)
 *
 * parv[1] = chan
 * parv[2] = level / mask / level
 * parv[3] = mask / dure / mask
 * parv[4] = dure / raison / raison
 * parv[5] = raison
 *
 * Syntax: BAN <chan> [level] <mask> [dure] [raison]
 */
void ban_cmd(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *cible;
	aBan *ban;
	aLink *join;
	char *chan = parv[1], raison[300] = {0}, *mask = NULL, *duree = NULL, m[300];
	int level = 0, l;
	long timeout = 0;

	/* Paramtrage des variables suivant le cas*/
	if(parc >= 3 && is_num(parv[2]))
	{ /* BAN chan level mask [dure] [raison]*/
		if(!strchr(parv[3], '@') || !strchr(parv[3],'!'))
		{
			syntax_cmd(nick, GetCmdIndexbyName("ban"));
			return;
		}
		mask = parv[3];
		level = atoi(parv[2]);
		if(parc >= 4)
		{ /* BAN chan level mask [dure ou raison]*/
			if(*parv[4] == '%')
			{ /* BAN chan level mask dure [raison]*/
				duree = ++parv[4];
				if(parc >= 5) parv2msgn(parc, parv, 5, raison, 300);
			}
			else parv2msgn(parc, parv, 4, raison, 300);
		}
	}
	else if(parc >= 3 && *parv[3] == '%')
	{ /* BAN chan mask dure [raison]*/
		if(!strchr(parv[2], '@') || !strchr(parv[2], '!'))
		{
			syntax_cmd(nick, GetCmdIndexbyName("ban"));
			return;
		}
		mask = parv[2];
		duree = ++parv[3];
		if(parc >= 4) parv2msgn(parc, parv, 4, raison, 300);
	}
	else
	{ /* BAN chan mask [raison]*/
		if(!strchr(parv[2], '@'))
		{
			syntax_cmd(nick, GetCmdIndexbyName("ban"));
			return;
		}
		if(!strchr(parv[2], '!'))
		{
			csntc(nick, "La syntaxe d'un mask de ban est : nick!user@host");
			return;
		}
		mask = parv[2];
		if(parc >= 3) parv2msgn(parc, parv, 3, raison, 300);
	}

	/* Nous passons maintenant au codage du ban en lui mme*/
	l = ChanLevelbyUserI(nick->user, chan);
	if(level < 0 || level > 500 || (l < level && !IsAnAdmin(nick->user)))
	{
		csntc(nick, "Le niveau doit tre compris entre 0 et 500, et ne doit pas tre plus grand que votre niveau sur le salon");
		return;
	}

	if(duree && !(timeout = convert_duration(duree)))
	{
		csntc(nick, "Dure de ban incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");
		return;
	}

	if(CNoBans(chaninfo))
	{
		csntc(nick, "Le NOBANS est activ sur %s", chan);
		return;
	}
	if(chaninfo->banlevel > l && !IsAnAdmin(nick->user))
	{
		csntc(nick, "Dsol mais le BanLevel est  %d.", chaninfo->banlevel);
		return;
	}

	/*recherche d'un mask couvrant le nouveau ban*/
	for(ban = chaninfo->banhead; ban && match(ban->mask, mask);ban = ban->next);
	if(ban)
	{
		csntc(nick, "\002%s\2 (%d) est dj dans la banlist de %s (vrifi par %s).", mask, ban->level, chan, ban->mask);
		return;
	}

	for(join= chaninfo->members;join;join=join->next)
	{
		cible=join->value.j->nick;
#ifdef HAVE_CRYPTHOST
		snprintf(m, 300, "%s!%s@%s", cible->nick, cible->ident, cible->crypt);
#else
		snprintf(m, 300, "%s!%s@%s", cible->nick, cible->ident, cible->host);
#endif
		if(!match(mask, m) && !check_protect(nick, cible, chan)) return;
	}
	csmode(chaninfo, "+b %s", mask);
	add_ban(chaninfo, mask, (raison[0] != '\0') ? raison : DEFRAISON, nick->user->nick, time(NULL), timeout, level);
	write_bans(chaninfo);
	return;
}



void kickban(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *cible;
	char *chan = parv[1], *ciblec=NULL, raison[300] = {0}, *mask = NULL, *duree = NULL;
	int level = 0, l = 0;
	unsigned long int timeout = 0;

	if(parc >= 3 && is_num(parv[2]))
	{ /* KICKBAN chan level nick [dure] [raison]*/
		ciblec=parv[3];
		level = atoi(parv[2]);
		if(parc >= 4)
		{ /* KICKBAN chan level pseudo [dure ou raison]*/
			if(*parv[4] == '%')
			{ /* KICKBAN chan level pseudo dure [raison]*/
				duree = ++parv[4];
				if(parc >= 5) parv2msgn(parc, parv, 5, raison, 300);
			}
			else parv2msgn(parc, parv, 4, raison, 300);
		}
	}
	else
	{
		ciblec=parv[2];

		if(parc >= 3 && *parv[3] == '%')
		{ /* KICKBAN chan mask dure [raison]*/
			duree = ++parv[3];
			if(parc >= 4) parv2msgn(parc, parv, 4, raison, 300);
		}
		else if(parc >= 3) parv2msgn(parc, parv, 3, raison, 300);
	}
	/* Nous passons maintenant au codage du ban en lui mme*/

	if(CNoBans(chaninfo))
	{
		csntc(nick, "Le NOBANS est activ sur %s", chan);
		return;
	}
	l = ChanLevelbyUserI(nick->user, chan);
	if(chaninfo->banlevel > l && !IsAnAdmin(nick->user))
	{
		csntc(nick, "Dsol mais le BanLevel est  %d.", chaninfo->banlevel);
		return;
	}

	if(!strcasecmp(cs.nick, ciblec))
	{
		csntc(nick, "Je suis pas sur que a soit prfrable pour \2%s\2...", chaninfo->chan);
		return;
	}

	if(!(cible = getnickbynick(ciblec)) || !(getjoininfo(cible, chaninfo->chan)))
	{
		csntc(nick, "\2%s\2 n'est pas dans %s", parv[2], chaninfo->chan);
		return;
	}
	if(!check_protect(nick, cible, chaninfo->chan)) return;

	if(level < 0 || level > 500 || (l < level && !IsAnAdmin(nick->user)))
	{
		csntc(nick, "Le niveau doit tre compris entre 0 et 500, et ne doit pas tre plus grand que votre niveau sur le salon");
		return;
	}

	if(duree && !(timeout = convert_duration(duree)))
	{
		csntc(nick, "Dure de ban incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");
		return;
	}

#ifdef HAVE_CRYPTHOST
	mask = getbanmask(cible->nick, cible->ident, cible->crypt, chaninfo->bantype);
#else
	mask = getbanmask(cible->nick, cible->ident, cible->host, chaninfo->bantype);
#endif
	csmode(chaninfo, "+b %s", mask);
	cskick(chaninfo->chan, cible->numeric, "(\2%s\2) %s", nick->user->nick, raison);
	add_ban(chaninfo, mask, (raison[0] != '\0') ? raison : DEFRAISON, nick->user->nick, time(NULL), timeout, level);
	write_bans(chaninfo);
	return;
}



void nickban(aNick *nick, aChan *chaninfo, int parc, char **parv)
{	aNick *cible;
	char *chan = parv[1], *ciblec = NULL, raison[300] = {0}, *mask = NULL, *duree = NULL;
	int level = 0, l = 0;
	long unsigned int timeout = 0;

	if(parc >= 3 && is_num(parv[2]))
	{ /* NICKBAN chan level nick [dure] [raison]*/
		ciblec = parv[3];
		level = atoi(parv[2]);
		if(parc >= 4)
		{ /* NICKBAN chan level pseudo [dure ou raison]*/
			if(*parv[4] == '%')
			{ /* NICKBAN chan level pseudo dure [raison]*/
				duree = ++parv[4];
				if(parc >= 5) parv2msgn(parc, parv, 5, raison, 300);
			}
			else parv2msgn(parc, parv, 4, raison, 300);
		}
	}
	else
	{
		ciblec = parv[2];

		if(parc >= 3 && *parv[3] == '%')
		{ /* NICKBAN chan mask dure [raison]*/
			duree = ++parv[3];
			if(parc >= 4) parv2msgn(parc, parv, 4, raison, 300);
		}
		else if(parc >= 3) parv2msgn(parc, parv, 3, raison, 300);
	}
	/* Nous passons maintenant au codage du ban en lui mme*/
	l = ChanLevelbyUserI(nick->user, chan);
	if(CNoBans(chaninfo))
	{
		csntc(nick, "Le NOBANS est activ sur %s", chan);
		return;
	}
	if(chaninfo->banlevel > l && !IsAnAdmin(nick->user))
	{
		csntc(nick, "Dsol mais le BanLevel est  %d.", chaninfo->banlevel);
		return;
	}

	if(!(cible = getnickbynick(ciblec)) || !(getjoininfo(cible, chaninfo->chan)))
	{
		csntc(nick, "\2%s\2 n'est pas dans %s", parv[2], chaninfo->chan);
		return;
	}
	if(!check_protect(nick, cible, chaninfo->chan)) return;


	if(level < 0 || level > 500 || (l < level && !IsAnAdmin(nick->user)))
	{
		csntc(nick, "Le niveau doit tre compris entre 0 et 500, et ne doit pas tre plus grand que votre niveau sur le salon");
		return;
	}


	if(duree && !(timeout = convert_duration(duree)))
	{
		csntc(nick, "Dure de ban incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");
		return;
	}

#ifdef HAVE_CRYPTHOST
	mask = getbanmask(cible->nick, cible->ident, cible->crypt, chaninfo->bantype);
#else
	mask = getbanmask(cible->nick, cible->ident, cible->host, chaninfo->bantype);
#endif
	csmode(chaninfo, "+b %s", mask);
	add_ban(chaninfo, mask, (raison[0] != '\0') ? raison : DEFRAISON, nick->user->nick, time(NULL), timeout, level);
	write_bans(chaninfo);
	return;
}

unsigned long int convert_duration(const char *duree)
{
	char *p = malloc(strlen(duree) + 1), *p1, *s = p;
	unsigned long int result = 0, t;

	strcpy(p, duree);
	p1 = p;

	while(*p1)
	{
		switch(*p1)
		{
			case 'H':
			case 'h':
						t = strtol(p, NULL, 10);
						result += 3600 * t;
						p = p1 + 1;
						break;
			case 'm':
						t = strtol(p, NULL, 10);
						result += 60 * t;
						p = p1 + 1;
						break;
			case 'Y':
			case 'y':
						t = strtol(p, NULL, 10);
						result += 31536000 * t;
						p = p1 + 1;
						break;
			case 'd':
			case 'j':
			case 'J':
			case 'D':
						t = strtol(p, NULL, 10);
						result += 3600 * 24 * t;
						p = p1 + 1;
						break;
		}
		p1++;
	}
	free(s);
	return result;
}

/*
 * banlist
 *
 * parv[1] = chan
 */
void banlist(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban, *tmp;
	int i = 0, m = 0, suppr = 0;
	char *c, buf[200];

	if(!parc)
	{
		syntax_cmd(nick, GetCmdIndexbyName("banlist"));
		return;
	}
	if(parc < 2) m = 1;
	c = parv[1];

	for(ban = chaninfo->banhead;ban;ban=ban->next)
	{
		if(ban->fin && (ban->fin + ban->debut) < time(NULL))
		{
			tmp = ban->next;
			del_ban(chaninfo, ban->mask);
			ban = tmp;
			suppr = 1;
			continue;
		}
		if(m || !match(parv[2], ban->mask))
		{
			i++;
			csntc(nick, "Mask du ban \002%s\2 Niveau: %d", ban->mask, ban->level);
			csntc(nick, "Ajout par \2%s\2 (%s)", ban->de, ban->raison);

			strncpy(buf, get_time(ban->debut), 200);
			csntc(nick, "Depuis \002%s\2, expire \002%s\2", buf, ban->fin ? get_time((ban->debut + ban->fin)) : "Jamais");
			csntc(nick, "-");
		}
	}

	if(i) csntc(nick, "Il y a un total de \002%d\2 ban%s sur \2%s\2.", i, (i > 1) ? "s" : "", c);
	else csntc(nick, "Aucun ban sur \2%s\2.", c);
	if(suppr) write_bans(chaninfo);
	return;
}

/*
 * clear_bans
 *
 * parv[1] = chan
 */
void clear_bans(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban, *tmp;
	char modes[6] = {0}, masks[450] = {0};
	int i = 0, nb = 0, n, from, minlevel, maxlevel;

	n = ChanLevelbyUserI(nick->user, chaninfo->chan);

	minlevel = getoption("-minlevel", parv, parc, 1, 1);
	maxlevel = getoption("-maxlevel", parv, parc, 1, 1);
	from = getoption("-from", parv, parc, 1, 0);

	for(ban = chaninfo->banhead;ban;ban=tmp)
	{
		tmp=ban->next;
		if((from && strcasecmp(parv[from], ban->de)) || (minlevel && ban->level < minlevel) || (maxlevel && ban->level > maxlevel)) continue;

		if(!IsAnAdmin(nick->user) && n < ban->level)
		{
			csntc(nick, "Le ban %s ne peut pas tre retir, votre niveau est trop faible", ban->mask);
			continue;
		}

		strcat(modes, "b");
		if(nb) strcat(masks, " ");
		strcat(masks, ban->mask);
		nb++;
		if(nb>4)
		{
			csmode(chaninfo, "-%s %s", modes, masks);
			modes[0] = '\0';
			masks[0] = '\0';
			nb = 0;
		}
		del_ban(chaninfo, ban->mask);
		i++;
	}
	if(nb) csmode(chaninfo, "-%s %s", modes, masks);

	if(i)
	{
		write_bans(chaninfo);
		if(i>1) csntc(nick, "Les \002%d\2 bans du salon %s ont t supprims.", i, chaninfo->chan);
		else csntc(nick, "Un ban trouv a t supprim.");
	}
	else csntc(nick, "Aucun ban trouv sur %s.", chaninfo->chan);
	return;
}

void unban(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban;
	char *b = parv[2];
	int n, trouve = 0;

	n = ChanLevelbyUserI(nick->user, chaninfo->chan);
	for(ban = chaninfo->banhead;ban;ban=ban->next)
	{
		if(!match(b, ban->mask))
		{
			if(!IsAnAdmin(nick->user) && n < ban->level)
			{
				csntc(nick, "Votre niveau est insuffisant pour retirer \002%s\2 (%d)", b, ban->level);
				continue;
			}
			csntc(nick, "Ban \002%s\2 sur %s supprim.", ban->mask, chaninfo->chan);
			csmode(chaninfo, "-b %s", ban->mask); /* quel oubli monstrueux :)*/
			del_ban(chaninfo, ban->mask);
			trouve = 1; /* on return pas, a permet ainsi d'unban plusieurs bans  la fois*/
		}
	}

	if(!trouve) csntc(nick, "Il n'y a pas de tel ban.");
	else write_bans(chaninfo);
	return;
}

void unbanme(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char h[200];
	aBan *bans;
	int n, i = 0, stillban = 0;

	n = ChanLevelbyUserI(nick->user, chaninfo->chan);
#ifdef HAVE_CRYPTHOST
	snprintf(h, 200, "%s!%s@%s", nick->nick, nick->ident, nick->crypt);
#else
	snprintf(h, 200, "%s!%s@%s", nick->nick, nick->ident, nick->host);
#endif
	for(bans=chaninfo->banhead;bans;bans=bans->next)
	{
		if(!match(bans->mask, h))
		{
			if(!IsAnAdmin(nick->user) && n < bans->level)
			{
				csntc(nick, "Votre niveau est insuffisant pour retirer \002%s\2 (%d)", bans->mask, bans->level);
				stillban = 1;
			}
			else
			{
				i++;
				csmode(chaninfo, "-b %s", bans->mask);
				csntc(nick, "Matching Ban \002%s\2 sur %s supprim.", bans->mask, chaninfo->chan);
				del_ban(chaninfo, bans->mask);
			}
		}
	}
	if(i) write_bans(chaninfo);
	csntc(nick, "%d Ban%s vous couvrant retir%s sur \2%s\2.", i, (i>1) ? "s" : "", (i>1) ? "s" : "", chaninfo->chan);
	if(!stillban && !getjoininfo(nick, chaninfo->chan)) putserv("%s %s %s :%s\n", cs.num, TOKEN_INVITE, nick->nick, chaninfo->chan);
	return;
}

char *getbanmask(const char *nick, const char *ident, const char *host, int bantype)
{
	static char mask[(HOSTLEN + 11)];
	char m[100], s[10];
	int a=0, b=0, c=0, d=0;

	if(bantype == 1 || !bantype) /*ban du type *!*ident@*.host */
	{
		if((sscanf(host, "%d.%s", &a,s) && strlen(s) == 8) || (is_ip(host) && sscanf(host, "%d.%d.%d.%d", &a,&b,&c,&d)))
         snprintf(m, 100, "%d.*",a);
        else snprintf(m, 100, "*%s", strchr(host, '.'));
        snprintf(mask, (HOSTLEN + 10), "*!*%s@%s", ident, m);
    }
    else if(bantype == 2) snprintf(mask, (HOSTLEN + 10), "*!*%s@*", ident); /*type : *!*ident@* */
   /* Note  celui ki fout un *!?%s@* : a ncessite strictement un char devant l'ident. Et si jamais l'ident
      est d, bha on y peut rien si *!*d@* se f ban et ke personne pourra avoir un ident finissant par d.
      On peut nanmoin fer un ban sur *!ident@* pour empecher a, mais dans ce cas si l'user a un ident en ~* faut
      demander au bot de le garder dans m_nick. Et si jamais l'user l'ajoute/l'enlve aprs, c comme si il changeait
      d'ident donc c pareil.  --Progs
   */
	else if(bantype == 3) snprintf(mask, (HOSTLEN + 10), "*%s*!*@*", nick); /*type : *nick*!*@* */
	else if(bantype) snprintf(mask, (HOSTLEN + 10), "*!*@%s", host); /*type : *!*@host */
	return mask;
}

aBan *is_ban(aNick *nick, aChan *chan)
{
	aBan *b = chan->banhead;
	char host[NICKLEN + USERLEN + HOSTLEN + 3];
#ifdef HAVE_CRYPTHOST
	char hostc[NICKLEN + USERLEN + HOSTLEN + 3];
	snprintf(hostc, NICKLEN + USERLEN + HOSTLEN + 2, "%s!%s@%s", nick->nick, nick->ident, nick->crypt);
#endif

	snprintf(host, NICKLEN + USERLEN + HOSTLEN + 2, "%s!%s@%s", nick->nick, nick->ident, nick->host);

	for(;b;b=b->next)
	{
		if(!match(b->mask, host)
#ifdef HAVE_CRYPTHOST
			|| !match(b->mask, hostc)
#endif
		) return b;
	}
	return NULL;
}
