/* src/divers.c - Diverses commandes
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"
#ifdef USEBSD
#include <sys/types.h>
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <netdb.h>

CVSINFO("$Id: divers.c,v 1.19 2003/04/28 22:35:20 romexzf Exp $");

extern struct ignore *ignorehead;
extern struct badnicks *badnhead;
extern struct badchans *badchead;
extern const int ConfFlag;

void uptime(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csntc(nick, "Les services sont en ligne depuis %s", duration((time(NULL) - tuptime)));
	return;
}

void ctcp_ping(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(parc) csntc(nick, "%cPING %s", CTCP_DELIM_CHAR, parv[1]);
	return;
}

void ctcp_version(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
    csntc(nick, "%cVERSION Services CoderZ %s  Inter System%c", CTCP_DELIM_CHAR, SPVERSION, CTCP_DELIM_CHAR);
    csntc(nick, "%cVERSION   Progs <Progs@Inter-System.Net>%c", CTCP_DELIM_CHAR, CTCP_DELIM_CHAR);
    csntc(nick, "%cVERSION   Cesar <Cesar@Inter-System.Net>%c", CTCP_DELIM_CHAR, CTCP_DELIM_CHAR);
    csntc(nick, "%cVERSION   kouak <kouak@kouak.org>%c", CTCP_DELIM_CHAR, CTCP_DELIM_CHAR);
    csntc(nick, "%cVERSION   http://coderz.inter-system.net%c", CTCP_DELIM_CHAR, CTCP_DELIM_CHAR);
	return;
}

void copyright(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
   csntc(nick, "Services CoderZ %s  Inter System", SPVERSION);
   return;
}

void lastseen(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;

	if(!(u=getuserinfo(parv[1])))
	{
		csntc(nick, NOSUCHUSER, parv[1]);
		return;
	}

	csntc(nick, "Dernier Login de %s il y a %s (%s)", u->nick, duration(time(NULL) - u->lastseend), get_time(u->lastseend));
	return;
}

void show_admins(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *user;
	struct nickinfo *nnick;
	char *what;

	csntc(nick, "  \002\037\0034Prsent\003\037   \037\0034Niveau\003\037    \037\0034Username\003\037       \037\0034Pseudo\003\037");
	user = userhead;
	while(user)
	{
		if(IsAnAdmin(user) && (!(ConfFlag & CF_HIDE_ADML4) || user->level < 4))
		{
			what = user->n ? "\002\0033OUI\003\002" : "\002\0034NON\003\002";
			if(!(nnick = user->n)) csntc(nick, "  %-3s      %2d         %-12s", what, user->level, user->nick, nnick->nick);
			else csntc(nick, "  %-3s      %2d         %-12s   \0032%s", what, user->level, user->nick, nnick->nick);
		}
		user = user->next;
	}
	return;
}

void show_ignores(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	struct ignore *i;
	int c = 0;
	time_t p = time(NULL);

	csntc(nick, "List des Ignors:");
	for(i=ignorehead;i;i=i->next)
	{
		if(i->exp <= p)
		{
			del_ignore(i->host);
			continue;
		}
		c++;
		csntc(nick,"\002%d\002. \002%s\002 Expire dans %s",c, i->host,duration((i->exp - p)));
	}
	if(!c) csntc(nick,"> Aucun actuellement.");
	return;
}

void list_badnicks(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	struct badnicks *current;

	csntc(nick, "\002\037Liste des pseudo illgaux :");
	for(current=badnhead;current;current=current->next) csntc(nick, "  \002%s\002: %s", current->chaine, (current->flag & CF_NOREG) ? "NoReg" : "Pseudo interdit");
	csntc(nick, "Les pseudos illgaux seront vrifis grce aux wildcards, comme pour les bans.");
    return;
}

void list_badchans(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	struct badchans *current;

	csntc(nick, "\002\037Liste des salons illgaux :");
	for(current=badchead;current;current=current->next) csntc(nick, "  \002%s\002 : %s", current->chaine, current->raison);
	csntc(nick, "Les salons illgaux sont vrifis avec des * et des ? comme les bans.");
    return;
}

void reop(aNick *nick, aChan *chan, int parc, char **parv)
{
	int i = 0;

	if(parc) i = 1;
	for(chan = chanhead;chan;chan=chan->next)
		if(!i || !match(parv[1], chan->chan)) putserv("%s MODE %s +o %s\n", bot.servnum, chan->chan, cs.num);
		/* bot.servnum -> cs.nick = fo pas dconner si le bot peut s'op tout seul c pas pour rien
		>> bah peut etre mais le reop c justement pour resynch les serveurs.. donc faut un HACK hin progs
		>> oui bha le pb c ke a flood grave en notice de HACK, donc autant un restart, mais l c lourd
		   mais je pige pas lors d'un desynch pk il en informe pas le bot ? lorske deux ircds se desynch, ils
		   savent tous les deux que tels users de le serveur se sont f deop, etc. y a une donne trait lors
		   d'un burst qui indik la raction ke va avoir lotre ircd ? Tout ce qu'on a c'est :
		    R - A BURST #Services 1036289958 AAC:o
		   qui suffit pour faire faire  l'ircd:
		    === (MODE) #Services +o gna par Irc.Inter-System.Net
		    === (MODE) #Services -tno Z par Irc.Inter-System.Net
		   j'imagine ke c grace au timestamp(qu'on a plus car del des bursts pis toute faon on s'en ouf)
		   Mais bon pour en revenir au dbut si c le bot ki s'op tout seul tous les serveurs le sauront
		   pour eux il sera op et fissa j'vois pas ce k'un op par serveur ajoute
		>> (tj par progs) bha j'ai bien pris cette option: le bot se souvient lors du merge des timestamp
		   de chaques chans et l'utilise pour deviner si on l'a deop ou pas. Il s'auto reop si il croit
		   qu'on l'a deop. Cette commande devient presque inutile
		   Le problme est que je sais pas o mettre le timestamp de bot.pchan,  moin que a soit dans cs */
#ifdef USE_WELCOMESERV
	putserv("%s MODE %s +oo %s %s\n", bot.servnum, bot.pchan, cs.num, ws.num);
#else
	putserv("%s MODE %s +o %s\n", bot.servnum, bot.pchan, cs.num);
#endif
	return;
}
