/* src/outils.c - Divers outils
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs_and_davidounet@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <stdarg.h>
#include <main.h>
#ifdef USEBSD
#include <sys/types.h>
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#ifdef HAVE_CRYPTHOST
#include <crypt.h>
#endif

CVSINFO("$Id: outils.c,v 1.43 2003/05/03 15:28:57 romexzf Exp $");

int dataS = 0;
extern struct Link *mainhub;

extern struct badchans *badchead;
extern struct badnicks *badnhead;

extern struct cs_commands cs_cmds[];

static const char *months[12] = {
  "Janvier", "Fevrier", "Mars", "Avril",
  "Mai", "Juin", "Juillet", "Aot",
  "Septembre", "Octobre", "Novembre", "Decembre"
};

static const char *weekdays[7] = {
  "Dimanche", "Lundi", "Mardi", "Mercredi",
  "Jeudi", "Vendredi", "Samedi"
};

struct Modes UMode[] = {
	{ 'i', N_INV },
	{ 'x', N_CRYPT },
	{ 'r', N_REG },
	{ 'o', N_OPER },
	{ 'k', N_SERVICE },
	{ 'a', N_ADM },
	{ 'p', N_GOD },
	{ 'f', N_FEMME },
	{ 'h', N_HOMME },
	{ 'g', N_DEBUG },
	{ 'w', N_WALLOPS },
	{ 'D', N_DIE },
	{ 'd', N_DEAF },
	{ 0, 0},

};

struct Modes CMode[] = {
	{ 'n', C_MMSG },
	{ 't', C_MTOPIC },
	{ 'i', C_MINV },
	{ 'l', C_MLIMIT },
	{ 'k', C_MKEY },
	{ 's', C_MSECRET },
	{ 'p', C_MPRIVATE },
	{ 'm', C_MMODERATE },
	{ 'c', C_MNOCTRL },
	{ 'C', C_MNOCTCP },
	{ 'O', C_MOPERONLY },
	{ 'r', C_MUSERONLY },
	{ 0, 0},
};

#ifdef HAVE_CRYPTHOST
char *hostprot(unsigned char *key, unsigned char *host)
{
	u_int32_t sum, digest[4];
	char tmp[5], ID[64], *ptr;
	static char newhost[HOSTLEN + 1];

	if(!host[0]%2)
	{
		MD2_CTX context;
		MD2Init(&context);
		MD2Update(&context,key,strlen(key));
		MD2Update(&context,host,strlen(host));
		MD2Final((void *) digest,&context);
	}
	else
	{
		MD5_CTX context;
		MD5Init(&context);
		MD5Update(&context,key,strlen(key));
		MD5Update(&context,host,strlen(host));
		MD5Final((void *) digest,&context);
	}

	sum=digest[0]+digest[1]+digest[2]+digest[3];

	sprintf(ID,"%X", sum);
	newhost[0] = '\0';

	if (!is_ip(host))
	{
		ptr = (char *) strchr(host, '.');
		strncpy(newhost, ID, HOSTLEN);
		strncat(newhost, ptr, HOSTLEN);
	}
	else
	{
		ptr = (char *) strrchr(host, '.');
		strcpy(tmp, host);
		strtok(tmp, ".");
		strncpy(newhost, tmp,5);
		strcat(newhost, ".");
		strncat(newhost, ID, 20);
		newhost[HOSTLEN]=0;
	}
	return (newhost);
}
#endif

/* Note: La fonction chan_check a t reprise du CS5.0  IRCore*/
int chan_check(const char *chan, aNick *nick)
{
	struct badchans *badchan;
	const char *p = chan;

	if(strlen(chan) > 100)
	{
		csntc(nick, "\2Le nom du salon est trop long (limite 100)\2");
		return 0;
	}
	if(strchr(chan, ',') || *p != '#')
	{
		csntc(nick, "[\2%s\2] n'est pas un nom de salon correct", chan);
		return 0;
	}
	if(!strcasecmp(bot.pchan, chan))
	{
		csntc(nick, "Salon non enregistrable.");
		return 0;
	}

	while(*p)
	{
		if(!isascii(*p))
		{
			csntc(nick, "\2Le nom du salon doit contenir aucun caractre spcial");
			return 0;
		}
		p++;
	}
	if((badchan = getbadchan(chan)))
	{
		csntc(nick, "Le salon \2%s\2 ne peut pas tre enregistr pour la raison suivante : \002%s\2", chan, badchan->raison);
		return 0;
	}
	return 1;
}

int is_num(const char *num)
{
   while(*num) if(!isdigit(*num++)) return 0;
   return 1;
}

int is_ip(const char *addr)
{
   while(*addr)
   {
	   if(!isdigit(*addr) && (*addr != '.')) return 0;
	   addr++;
   }
   return 1;
}

void strupr(char *buf)
{
	while(*buf)
	{
	   *buf = toupper(*buf);
	   buf++;
	}
	*buf = '\0';
}

void strlwr(char *buf)
{
	while(*buf)
	{
		*buf = tolower(*buf);
		buf++;
	}
	*buf = '\0';
}

void conv_char(char *chan, char *string)
{
	char val[10], *ptr = chan;
	strcpy(string, "");

	while(*ptr != '\0')
	{
		if(!isalnum(*ptr)) sprintf(val, "%%%03o", *ptr);
		else sprintf(val, "%c", *ptr);
		strcat(string, val);
		ptr++;
	}
	strcat(string, "\0");
	return;
}

void strip_newline(char *string)
{
   char *p = string;
   while(*p && *p != '\n' && *p != '\r') p++;
   *p = '\0';
}

int ChanLevelbyUserI(anUser *user, const char *chan)
{
	anAccess *acces=NULL;
	if((acces=GetAccessIbyUserI(user,chan))) return acces->level;
	return 0;
}

aJoin *getjoininfo(aNick *nick, const char *chan)
{
	aJoin *join = nick->joinhead;
	for(;join && strcasecmp(chan, join->channel);join=join->next);
	return join;
}

aChan *getchaninfo(const char *chan)
{
	aChan *current = chanhead;
	for(;current && strcasecmp(chan,current->chan);current=current->next);
	return current;
}

anUser *getuserinfo(const char *nick)
{
	anUser *current = userhead;
	for(;current && strcasecmp(nick,current->nick);current=current->next);
	return current;
}

aNick *getnickbynick(const char *nick)
{
	aNick *current = nickhead;
	for(;current && strcasecmp(nick,current->nick); current = current->next);
	return current;
}

struct badnicks *getbadnick(const char *chan)
{
   struct badnicks *current = badnhead;
   for(;current;current=current->next) if(!match(current->chaine, chan)) return current;
   return NULL;
}

int IsBadNick(const char *chan)
{
   struct badnicks *current;
   for(current = badnhead;current;current=current->next) if(!match(current->chaine, chan)) return 1;
   return 0;
}

struct badchans *getbadchan(const char *chan)
{
   struct badchans *current;
   for(current=badchead;current && match(current->chaine, chan);current = current->next);
   return current;
}

char *IsOwner(const char *nick, const char *chan)
{
	anAccess *access;
	anUser *user = getuserinfo(nick);

	if(!user) return NULL;
	if((access=GetAccessIbyUserI(user,chan)) && access->level == 500) return access->chan;
	return NULL;
}

char *IsAnOwner(const char *nick)
{
	anAccess *access;
	anUser *user = getuserinfo(nick);

	if(!user) return NULL;
	for(access = user->accesshead;access;access = access->next) if(access->level == 500) return access->chan;
	return NULL;
}

int IsRegChan(const char *chan)
{
	aChan *c = getchaninfo(chan);
	return c ? 1 : 0;
}

int IsRegNick(const char *nick)
{
   anUser *user = getuserinfo(nick);
   return user ? 1 : 0;
}

aNick *num2nickinfo(const char *num)
{
   aNick *nick = nickhead;

   for(;nick;nick=nick->next) if(!strcmp(num, nick->numeric)) return nick;
   return NULL;
}

char *num2nick(const char *num)
{
   aNick *nick;

   for(nick = nickhead;nick;nick=nick->next) if(!strcmp(num, nick->numeric)) return nick->nick;
   return NULL;
}

char *nick2num(const char *nick)
{
   aNick *current;

   for(current = nickhead;current;current=current->next) if(!strcasecmp(nick, current->nick)) return current->numeric;
   return NULL;
}

anAccess *GetAccessIbyUserI(anUser *user, const char *chan)
{
	anAccess *a=NULL;
	for(a = user->accesshead ;a && strcasecmp(a->chan,chan) ; a = a->next);
	return (a && !(a->flag & A_WAITACCESS)) ? a : NULL;
}

/*
 * getoption:
 * recherche dans un tableau char ** (de 'parc' entres)
 * l'expression "opt"  partir de l'lment 'deb'
 * -si exp_int est > 0:
 *  cela veut dire qu'on attend une valeur numerique apres le char 'opt'
 *  si c'est le cas, la fct renvoit la valeur, sinon 0
 * -si exp_int est < 0:
 *  cela veut dire que l'on recherche uniquement si 'opt' est dans parv,
 *  si c'est le cas, retourne 1
 * -si exp_int est = 0:
 *  cela veut dire que l'on attend une string apres le 'opt',
 *  si c'est le cas retourne l'index de cette string dans parv, sinon 0.
 *
 */

int getoption(char *opt, char **parv, int parc, int deb, int exp_int)
{
	for(;deb<=parc;deb++)
	{
		if(!strcasecmp(opt,parv[deb]))
		{
			if(exp_int < 0) return 1;
			if(++deb<=parc)
			{
				if(is_num(parv[deb]) && exp_int) return atoi(parv[deb]);
				else if(!exp_int) return deb;
			}
			break;
		}
	}
	return 0;
}

struct Link *num2servinfo(const char *num)
{
	struct Link *link;

	for(link=mainhub;link;link=link->next)
	{
#ifdef N2K
		if(!strncmp(num,link->num,2)) return link;
#else
		if(!strncmp(num,link->num,1)) return link;
#endif
	}
	return NULL;
}

struct Link *GetLinkIbyServ(const char *serv)
{
	struct Link *link;
	for(link=mainhub;link && strcasecmp(link->serv,serv);link=link->next);
	return link;
}

static void log(const char *filename, const char *format, ...)
{
   va_list arglist;
   FILE *fp;
   if(!(fp = fopen(filename, "a"))) return;
   va_start(arglist, format);
   vfprintf(fp, format, arglist);
   va_end(arglist);
   fclose(fp);
   return;
}

void parv2msgn(unsigned int parc, char **parv, unsigned int base, char *buf, unsigned int size)
{
	unsigned int i, taillec, towrite;

	strcpy(buf,"");
	size--;
	for(i=base;i<=parc;i++)
	{
		if(!size) return;
		taillec=strlen(parv[i]);
		if(i>base)
		{
			strcat(buf, " ");
			size--;
		}
		if(size < taillec)
		{
			towrite=size;
			size=0;
		}
		else
		{
			towrite = taillec;
			size -= taillec;
		}
		strncat(buf, parv[i], towrite);
	}
	strcat(buf, "\0");
	return;
}

char *parv2msg(unsigned int parc, char **parv, unsigned int base, unsigned int size)
{
	unsigned int i, taillec, towrite;
	static char buf[500];
	buf[0] = '\0';

	size--;
	for(i=base;i<=parc;i++)
	{
		if(!size) return buf;
		taillec=strlen(parv[i]);
		if(i>base)
		{
			strcat(buf, " ");
			size--;
		}
		if(size < taillec)
		{
			towrite=size;
			size=0;
		}
		else
		{
			towrite = taillec;
			size -= taillec;
		}
		strncat(buf, parv[i], towrite);
	}
	strcat(buf, "\0");
	return buf;
}

int ToggleFlag(int flags, int flag)
{
	if(flags & flag) flags &= ~flag;
	else flags |= flag;
	return flags;
}

char *duration(int i)
{
	int mins, hours, days;
	static char dur[200];
	char tmp2[30];

	if(i >= 60)
	{
		mins = i / 60;
		i -= 60 * mins;
	}
	else mins = 0;
	if(mins >= 60)
	{
		hours = mins / 60;
		mins -= 60 * hours;
	}
	else hours = 0;
	if(hours >= 24)
	{
		days = hours / 24;
		hours -= 24 * days;
	}
	else days = 0;
	strcpy(dur, "");

	if(days) {
	  sprintf(tmp2, "\002%d\2 jours", days);
	  strcat(dur, tmp2);
	}
	if(hours) {
	  if(days) strcat(dur, " ");
	  sprintf(tmp2, "\002%d\2 heures", hours);
	  strcat(dur, tmp2);
	}
	if(mins) {
	  if(days||hours) strcat(dur, " ");
	  sprintf(tmp2, "\002%d\2 minutes", mins);
	  strcat(dur, tmp2);
	}
	if(!((mins || hours || days) && !i)) {
	  if(days||hours||mins) strcat(dur, " et ");
	  sprintf(tmp2, "\002%d\2 secondes", i);
	  strcat(dur, tmp2);
	}
	strcat(dur, "\0");
	return dur;
}

char *get_time(long ttime)
{
  register struct tm *lt;
  static char time[70];
  time_t clock;

  clock = ttime;
  lt = localtime(&clock);

  snprintf(time, 70, "%s %d %s %d %02d:%02d:%02d",
      weekdays[lt->tm_wday], lt->tm_mday, months[lt->tm_mon],
      1900 + lt->tm_year, lt->tm_hour, lt->tm_min, lt->tm_sec);

  return time;

}

char *GetPrefix(aNick *nick)
{
	static char pref[12];
	pref[0] = '\0';

	if((nick->flag & N_OPER)) strcat(pref, "\2\0037*\2\003");
	if(nick->user)
	{
		if(IsAnAdmin(nick->user)) strcat(pref, "\00312^\003");
		else strcat(pref, "\0035!\003");
	}
	strcat(pref, "\0");
	return pref;
}

char *GetChanPrefix(aNick *nick, aJoin *join, int colors)
{
	static char pref[14];
	pref[0] = '\0';

	if((nick->flag & N_OPER)) strcat(pref, colors ? "\2\0037*\2\003" : "*");
	if(nick->user)
	{
		if(IsAnAdmin(nick->user)) strcat(pref, colors ? "\00312^\003" : "^");
		else strcat(pref, colors ? "\0035!\003" : "!");
	}
	if(join->is_op) strcat(pref, "@");
	if(join->is_voice) strcat(pref, "+");

	strcat(pref, "\0");
	return pref;
}

char *GetUserOptions(anUser *user)
{
	static char option[120];
	option[0] = '\0';

	if(UNopurge(user)) strcat(option, "NoPurge ");
	if(UNoMemo(user)) strcat(option, "NoMemo ");
	if(UFirst(user)) strcat(option, "Registering ");
	if(UOubli(user)) strcat(option, "Oubli ");
	if(UWantX(user)) strcat(option, "VirtualHost ");
	if(UWantDrop(user)) strcat(option, "Droping ");
#ifdef USE_NICKSERV
	if(UProtect(user))
	{
		strcat(option, "Protection");
		if(UKill(user)) strcat(option, "(KILL) ");
		else strcat(option, "(ChNick) ");
	}
#endif
	if(user->flag & U_PREJECT) strcat(option, "Accs(Rejet)");
	if(user->flag & U_POKACCESS) strcat(option, "Accs(Ask)");
	if(user->flag & U_PACCEPT) strcat(option, "Accs(Accepte)");

	if(option[0] == '\0') strcat(option, "Aucune");
	strcat(option, "\0");
	return option;
}

char *GetAccessOptions(anAccess *acces)
{
	static char options[100];
	options[0] = '\0';

	if(AOp(acces)) strcat(options, " autoop");
	if(AProtect(acces)) strcat(options, " protg");
	if(AVoice(acces)) strcat(options, " autovoice");
	if(ASuspend(acces)) strcat(options, " suspendu");
	if(options[0] == '\0') strcpy(options, " Aucune");
	else strcat(options, "\0");
	return options;
}

int checkpass(const char *pass, anUser *user)
{
	char p[15];
	strncpy(p, pass, 15);
	if(strcmp(mkpass(p, "lala"), user->passwd)) return 0;
	return 1;
}

void putlog(char *fichier, const char *pattern, ...)
{
	char buf[512] = {0};
	va_list vl;

	if(!fichier || fichier == '\0') return;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);
	log(fichier, "[%s] %s", get_time(time(NULL)), buf);
}

void putserv(const char *pattern, ...)
{
	char buf[512] = {0};
	int len;
#ifdef DEBUG
	char tmp[10], tmp2[10];
#endif
	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);

	len = strlen(buf);
	send(bot.sock, buf, len, 0);
	dataS += len;

#ifdef DEBUG
	snprintf(tmp, 10, "%s %s", bot.servnum, TOKEN_PONG);
	snprintf(tmp2, 10, "%s %s", bot.servnum, TOKEN_PING);
#ifdef N2K
	if(strncmp(buf, tmp, 4) && strncmp(buf, tmp2, 4))
#else
	if(strncmp(buf, tmp, 5) && strncmp(buf, tmp2, 5))
#endif
    putlog(LOG_PARSES, "S - %s",buf);
#endif
	return;
}

void csntc(aNick *nick, const char *pattern, ...)
{
	char buf[512] = {0};
	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);

	putserv("%s %s %s :%s\n", cs.num, TOKEN_NOTICE, nick->numeric, buf);
	return;
}

void putchan(const char *buf)
{
	time_t timer;
	struct tm *ntime;
	char buftime[20];

	time(&timer);
	ntime=localtime(&timer);
	snprintf(buftime, 20, "%02d:%02d:%02d", ntime->tm_hour, ntime->tm_min, ntime->tm_sec);

	putserv("%s %s %s :[%s] %s\n", cs.num, TOKEN_PRIVMSG, bot.pchan, buftime, buf);
	return;
}

void csmode(aChan *chan, const char *pattern, ...)
{
	char buf[512] = {0}, mode[50] = {0}, keyc[24] = {0}, limitc[24] = {0}, *tmp, *lp, *kp;
	int arg = 0, w = 1, cur = 0, limit = 0, key = 0;

	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);
	putserv("%s %s %s %s\n", cs.num, TOKEN_MODE, chan->chan, buf);

	tmp = buf;
	kp = keyc;
	lp = limitc;
	while(*tmp && *tmp != ' ')/*parsge des modes & filtrage des +ovb & obtention du n des param de key/limite*/
	{
		switch(*tmp)
		{
			case 'o':
			case 'v':
			case 'b':
				arg++;
				break;
			case 'k':
				key = ++arg;
				mode[cur++] = *tmp;
				break;
			case 'l':
				if(w) limit = ++arg;
				mode[cur++] = *tmp;
				break;
			case '-': w = 0;
			case '+': w = 1;
			default:
				mode[cur++] = *tmp;
				break;
		}
		tmp++;
	}
	tmp++; /* on saute l'espace*/
	arg = 1;
	while(*tmp)
	{
		if(*tmp == ' ')
		{
			arg++;
			tmp++;
		}
		else if(arg == limit) while(*tmp && *tmp != ' ') *lp++ = *tmp++; /* on lit le paramtre corrsspondant*/
		else if(arg == key) while(*tmp && *tmp != ' ') *kp++ = *tmp++;
		else tmp++;
	}
	/*++lp = '\0'; fin des char[]*/
	*++kp = 0;

	parse_cmode(chan, mode, limitc, key ? keyc : ((chan->key[0] != '\0') ? chan->key : NULL));
	return;
}

void cstopic(aChan *chan, const char *pattern, ...)
{
  char buf[TOPICLEN + 1] = {0};
  va_list vl;
  va_start(vl, pattern);
  vsnprintf(buf, TOPICLEN, pattern, vl);
  va_end(vl);
  putserv("%s %s %s :%s\n", cs.num, TOKEN_TOPIC, chan->chan, buf);
  strcpy(chan->topic, buf);
}

void cskick(const char *chan, const char *vict, const char *pattern, ...)
{
  char buf[512] = {0};
  va_list vl;
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv("%s %s %s %s :%s\n", cs.num, TOKEN_KICK, chan, vict, buf);
}

void cswallops(const char *pattern, ...)
{
  char buf[300] = {0};
  va_list vl;
  va_start(vl, pattern);
  vsnprintf(buf, 300, pattern, vl);
  va_end(vl);
  putserv("%s %s :%s\n", cs.num, TOKEN_WALLOPS, buf);
}

void csjoin(aChan *chan)
{
	aLink *lp;
	aNick *n;
	aJoin *j;

	putserv("%s %s %s\n", cs.num, TOKEN_JOIN, chan->chan);
	for(n=nickhead;n;n=n->next)
		for(j=n->joinhead;j;j=j->next)
		{
			if(!(strcasecmp(chan->chan, j->channel)))
			{	 /* on a trouv le bon chan hh */
				lp = (aLink *) calloc(1, sizeof(aLink));
				lp->value.j = j;
				lp->next = chan->members;
				chan->members = lp;
				break;
			}
		}
}

void cspart(const char *chan, const char *pattern, ...)
{
	char buf[300] = {0};
	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 300, pattern, vl);
	va_end(vl);
	putserv("%s %s %s :%s\n", cs.num, TOKEN_PART, chan, buf);
}

void cs_account(aNick *nick, const char *account)
{
#ifdef N2K
	putserv("%s %s %s %s\n", bot.servnum, TOKEN_ACCOUNT, nick->numeric, account ? account : "");
#else
	putserv(":%s %s %s %s\n", cs.nick, TOKEN_ACCOUNT, nick->numeric, account ? account : "");
#endif
	return;
}

char *make_servnum(int c)
{
	static char res[3];
	const char base64[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789[]";
#ifndef N2K
	int i = 0;

	memset(res, 0, 3);
	for (; i>=0; c = c >> 6, i--) res[i] = base64[c & 63];
#else
	res[0] = base64[c / 64];
	res[1] = base64[c % 64];
	res[2] = '\0';
#endif

	return res;
}

int resolve(const char *host)
{
	struct hostent *he = gethostbyname(host);

	if(!he) return(inet_addr(host));
	return (*(int *)he->h_addr);
}


int check_protect(aNick *executant, aNick *execute, const char *chan)
{

   anAccess *access1, *access2;

   if(!strcasecmp(executant->nick, execute->nick)) return 1;
   else if((execute->flag & N_GOD) && !(executant->flag & N_OPER) && !IsAnAdmin(executant->user))
   {
	   csntc(executant, "L'user \2%s\2 est +p", execute->nick);
	   return 0;
   }
   else if(execute->user == NULL) return 1;
   else if(IsAnAdmin(executant->user) && executant->user->level > execute->user->level) return 1; /*l'executant est un admin de level suprieur a l'execut*/
   else if(IsAnAdmin(executant->user) && IsAnAdmin(execute->user) && executant->user->level <= execute->user->level && execute->user->level < 4)
   {
      csntc(executant, "L'user \2%s\2 est un Admin de niveau suprieur ou gal au votre.", execute->nick);
	  return 0; /*ce sont tous les 2 des admins mais executant est inferieur a execut*/
   }
   else if(!IsAnAdmin(executant->user) && IsAnAdmin(execute->user))
   {
      csntc(executant, "L'user \2%s\2 est un Administrateur des Services.", execute->nick);
	  return 0; /*executant n'est pas admin mais execute est admin*/
   }

    /*optimisation: on check dabord l'acces de l'execut!*/
   if(!(access2=GetAccessIbyUserI(execute->user, chan))) return 1; /*execut = sans acces*/
   if(!access2->flag & A_PROTECT) return 1; /*ca vaut pas le coup de boucler sil est pas protect..*/

   /*on boucle les access de l'executant pour voir si on en trouve 1 sur le chan*/
   if(!(access1=GetAccessIbyUserI(executant->user, chan))) return 0; /*ne doit jamais arriver, voudrait dire que executant n'est pas un admin et n'a pas d'access sur le chan*/

   if(access1->level > access2->level) return 1; /*on verifie si executant a un level superieur a execute, ou si execute n'est pas protect;*/

   csntc(executant, "L'user \2%s\2 est protg ou a un access suprieur au votre sur \2%s\2.", execute->nick, chan);
   return 0; /*arrive pour tous les autres cas*/

}

void parse_umode(aNick *nick, char *p)
{
	int w = 1, i;

	while(*p)
	{
		if(*p == '+') w = 1;
		else if(*p == '-') w = 0;
		else
		{
			for (i = 0;UMode[i].mode;i++)
				if(*p == UMode[i].mode)
				{
					if(w) nick->flag |= UMode[i].value;
					else nick->flag &= ~UMode[i].value;
					break;
				}
		}
		p++;
	}
	return;
}

char *GetModes(aNick *nick)
{
	static char mode[15];
	int i = 0, j = 0;
	for(;UMode[i].mode;i++) if((nick->flag & UMode[i].value)) mode[j++] = UMode[i].mode;
	mode[j] = '\0';
	return mode;
}

void parse_cmode(aChan *chan, char *p, const char *limit, const char *key)
{
	int w = 1, i;

	while(*p)
	{
		if(*p == '+') w = 1;
		else if(*p == '-') w = 0;
		else
		{
			for (i = 0;CMode[i].mode;i++)
				if(*p == CMode[i].mode)
				{
					if(w) chan->flag |= CMode[i].value;
					else chan->flag &= ~CMode[i].value;
					break;
				}
		}
		p++;
	}
	chan->limit = (chan->flag & C_MLIMIT) ? ((limit) ? atoi(limit) : chan->limit) : 0;
	if((chan->flag & C_MKEY) && key) strncpy(chan->key, key, 24);
	else chan->key[0] = '\0';
	return;
}

char *GetCModes(aChan *chan)
{
	static char mode[40];
	char limit[15] = {0};
	int i = 0, j = 0;
	for(;CMode[i].mode;i++) if((chan->flag & CMode[i].value)) mode[j++] = CMode[i].mode;
	mode[j] = '\0';
	if((chan->flag & C_MLIMIT))
	{
		snprintf(limit, 15, " %d", chan->limit);
		strcat(mode, limit);
	}
	strcat(mode, "\0");
	return mode;
}

int IsValidNick(char *p)
{
	if(*p=='-' || isdigit(*p)) return 0;
	while(*p)
	{
		if(!isalpha(*p) && !isdigit(*p) && *p != '\\' && *p != '-' && *p != '{' && *p != '}'
		     && *p != '[' && *p != ']' && *p != '_' && *p != '|' && *p != '`') return 0;
			p++;
	}
	return 1;
}

int IsValidMail(char *p)
{
	int fa = 0, fd = 0, ispoint = 0;
	if(*p == '-' || *p == '_' || *p == '.' || isdigit(*p) || *p == '@') return 0;
	while(*p)
	{
		if(*p=='@')
		{
			if(!fa) fa=1;
			else return 0;
		}
		if(fa && *p != '.') ispoint=0;
		if(fa && *p == '.')
		{
			if(ispoint) return 0; /* le char d'avant etait un point, 2 points successifs dans l'url interdits */
			fd=1;
			ispoint=1;
		}
		if(!isalpha(*p) && !isdigit(*p) && *p != '.' && *p != '-' && *p != '_' && *p!='@') return 0;
		p++;
	}
	if(fa && fd) return 1;
	return 0;
}

int GetCmdIndexbyName(const char *name)
{
	unsigned int i = 0;

	for(;cs_cmds[i].cmd[0]!='\0';i++) if(!strcasecmp(name,cs_cmds[i].realcmd)) return i;
	return -1;
}

int GetCurCmdIndexbyName(const char *name)
{
	unsigned int i = 0;

	for(;cs_cmds[i].cmd[0]!='\0';i++) if(!strcasecmp(name,cs_cmds[i].cmd)) return i;
	return -1;
}

char *RealCmd(const char *cmd)
{
	static char rcmd[15];
	int i = GetCmdIndexbyName(cmd);

	rcmd[0] = '\0';
	if(i >= 0) strcpy(rcmd, cs_cmds[i].cmd);
	return rcmd;
}

anUser *ParseNickOrUser(aNick *nick, char *arg)
{
	anUser *user;

	if(*arg == '%') /* recherche sur nick actif! */
	{
		aNick *n = getnickbynick(++arg);
		if(*arg == 0)
		{
			csntc(nick, "Veuillez spcifier un nick aprs '%%' : COMMANDE %%nick-connect [args..]");
			return NULL;
		}
		if(!n)
		{
			csntc(nick, NOTCONNECTED, arg);
			return NULL;
		}
		if(!n->user)
		{
			csntc(nick, "%s n'est actuellement logu sur aucun UserName.", arg);
			return NULL;
		}
		return n->user;
	}
	if(!(user = getuserinfo(arg)))
	{
		csntc(nick, NOSUCHUSER, arg);
		return NULL;
	}
	return user;
}

void File_UserDel(char *nick)
{
	char conv[50] = {0}, path[150];

	conv_char(nick, conv);
	strlwr(conv);
	snprintf(path, 150, "database/usr/%s", conv);
	remove(path);
	return;
}
