/* src/serveur.c - Fonctions serveur
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <unistd.h>
#include "main.h"
#ifdef USEBSD
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/socket.h>
#ifdef TESTBOT
#include "web2cs.h"
#endif

CVSINFO("$Id: serveur.c,v 1.65 2003/04/28 22:35:21 romexzf Exp $");

struct Link *mainhub;
int dataQ = 0;
int burst;
int registered;
int want_to_quit;
extern const int STS;
extern const int kill_interval;
extern const char *pasdeperm;
extern const int ConfFlag;
extern int lastresetparse;
extern int lastresetlogs;

extern struct cs_commands cs_cmds[];

extern void choose_welcome(char *num);

void m_ping(int, char **);
void m_privmsg(int, char **);
void m_pass(int, char **);
void m_kick(int, char **);
void m_part(int, char **);
void m_join(int, char **);

void m_mode(int, char **);
void m_eob(int, char **);
void m_squit(int, char **);
void m_server(int, char **);
void m_create(int, char **);
void m_nick(int, char **);
void m_quit(int, char **);
void m_burst(int, char **);
void m_away(int, char **);
#ifndef N2K
void m_sethost(int, char **);
void m_setident(int, char **);
void m_account(int, char **);
#endif
void m_whois(int, char **);
void m_kill(int, char **);
void m_topic(int, char **);

struct aMsg msgtab[] = {
  {TOKEN_PRIVMSG, m_privmsg},
  {TOKEN_PING, m_ping},
  {TOKEN_NICK, m_nick},
  {TOKEN_JOIN, m_join},
  {TOKEN_PART, m_part},
  {TOKEN_MODE, m_mode},
  {TOKEN_KICK, m_kick},
  {TOKEN_CREATE, m_create},
  {TOKEN_QUIT, m_quit},
  {TOKEN_BURST, m_burst},
  {TOKEN_AWAY, m_away},
  {TOKEN_SERVER, m_server},
#ifdef N2K
  {"SERVER", m_server},
#endif
  {TOKEN_SQUIT, m_squit},
  {TOKEN_TOPIC, m_topic},
  {TOKEN_KILL, m_kill},
  {TOKEN_WHOIS, m_whois},
#ifndef N2K
  {TOKEN_SETHOST, m_sethost},
  {TOKEN_SETIDENT, m_setident},
  {TOKEN_ACCOUNT, m_account},
#endif
  {TOKEN_PASS, m_pass},
  {TOKEN_EOB, m_eob},
  {NULL, NULL}
};

#ifndef N2K
/*
 * m_account
 *
 * parv[0] = sender (i don't care about)
 * parv[1] = numeric
 * parv[2] = login
 */
void m_account(int parc, char **parv)
{
	aNick *nick;
	anUser *user;
	char *login = NULL;

	if(parc != 3) return;
	if(!(nick=num2nickinfo(parv[1]))) return;

	login = parv[2];
	if(*login == ':') login++;

	if(!(user=getuserinfo(login)))
	{
		struct Link *link = num2servinfo(parv[0]);
		cswallops("ACCOUNT inconu (%s) pour %s depuis %s", login, nick->nick, link->serv);
		cs_account(nick, NULL); /* desacounter :)*/
		return;
	}
	if(user->n)
	{
		struct Link *link = num2servinfo(parv[0]);
		cswallops("ACCOUNT DESYNCH (%s) pour %s depuis %s", login, nick->nick, link->serv);
		cs_account(nick, NULL); /* desacounter :)*/
		return;
	}

	nick->user = user;
	nick->user->n = nick;
	nick->user->lastseend = time(NULL);
	csntc(nick, "Vous avez t \2ridentifi\2");
	if(UProtect(user)) del_killinfo(nick->nick, nick->numeric); /* arret du kill */
	return;
}
#endif

/*
 * m_mode
 *
 * parv[0] = who
 * parv[1] = chan
 * parv[2] = modes
 * parv[3->parc-1] = rest
 */
void m_mode(int parc, char **parv)
{
	aLink *j;
	anAccess *a = NULL, *a2 = NULL;
	aNick *nick, *nicktemp = NULL;
	aChan *ch = NULL;
	aJoin *join;
	char *modes = NULL, *param = NULL, *key = NULL, *limit = NULL;
	int what = 1, i = 2, l = 0;

	if(parc < 2) return;
	if(parv[1][0] == '#')
	{
		ch=getchaninfo(parv[1]);

		modes = parv[2];
		while (modes && *modes)
		{
			switch (*modes)
			{
				case '+':
					what = 1;
					break;
				case '-':
					what = 0;
					break;
				case 'o':
					do
					{
						i++;
						param = parv[i];
					}
					while(!*param || !param);

					if(ch) /* salon reg */
					{
						if(!what) /* -o */
						{
							if(!strcmp(cs.num, param)) /* deop du CServ */
							{
								csmode(ch, "+o %s", cs.num);
								break;
							}
							/* protection du deop des admins /user auths */
							if((nicktemp = num2nickinfo(param)) && nicktemp->user &&
#ifdef N2K
							(nick=num2nickinfo(parv[0]))
#else
							(nick=getnickbynick(parv[0]))
#endif
							&& nick != nicktemp && (IsAnAdmin(nicktemp->user) || (a=GetAccessIbyUserI(nicktemp->user, ch->chan))))
							{
								if(!nick->user || (nicktemp->user->level > nick->user->level ||
								(a && AProtect(a) && !ASuspend(a) && (a2=GetAccessIbyUserI(nick->user, ch->chan)) && !ASuspend(a2) && a2->level <= a->level)))
								{
									csmode(ch, "+o %s", nicktemp->numeric);
									break;
								}
							}
						}
						else /* +o */
						{
							nicktemp = num2nickinfo(param);
							if(CNoOps(ch) && nicktemp && !(nicktemp->flag & N_OPER))
							{
								csmode(ch, "-o %s", param);
								break;
							}
							if(CStrictOp(ch) && nicktemp && (!nicktemp->user || (!(a=GetAccessIbyUserI(nicktemp->user, ch->chan)) || ASuspend(a))))
							{
								csmode(ch, "-o %s", param);
								break;
							}
						}
						/* mise  jour des modes */
						for(j=ch->members;j && strcasecmp(j->value.j->nick->numeric, param);j=j->next);
						if(j) j->value.j->is_op = what;

					} /* mise  jour direct dans nick->join */
					else if((join = getjoininfo((nicktemp = num2nickinfo(param)), parv[1]))) join->is_op = what;

					break;

				case 'v':
					do
					{
						i++;
						param = parv[i];
					}
					while(!*param || !param);

					j=NULL;
					join=NULL;
					if(ch)
					{
						for(j=ch->members;j;j=j->next)
						if(!strcasecmp(j->value.j->nick->numeric, param)) break;
						if(j) j->value.j->is_voice = what;
					}
					else if((join=getjoininfo((nicktemp=num2nickinfo(param)), parv[1]))) join->is_voice = what;
					break;

				case 'b':
					do  /*l on modifie 'i' pour qu'il passe au paramtre de ce mode, pour pas ke celui du suivant soit celui ci, hein cesar*/
					{
						i++;
						param = parv[i];
					}
					while(!*param || !param);
					if(!what || !ch) break;
					if(!CNoBans(ch))
					{
						char h[(NICKLEN + USERLEN + HOSTLEN + 3)]; /* 3 = ! @ \0 */
#ifdef N2K
						if(!(nicktemp=num2nickinfo(parv[0]))) break;
#else
						if(!(nicktemp=getnickbynick(parv[0]))) break;
#endif
						l = nicktemp->user ? ChanLevelbyUserI(nicktemp->user, parv[1]) : 0;
						for(j=ch->members;j;j=j->next)
						{
							nick=j->value.j->nick;
							snprintf(h, (NICKLEN + USERLEN + HOSTLEN), "%s!%s@%s", nick->nick, nick->ident,
#ifdef HAVE_CRYPTHOST
							nick->crypt
#else
							nick->host
#endif
							);
							if(match(param, h) || !nick->user) continue;
							for(a=nick->user->accesshead;a && strcasecmp(a->chan, ch->chan);a=a->next);
							if(a && AProtect(a)) break;
							else a=NULL;
						}
					}
					if(CNoBans(ch) || ((ch->banlevel > l || (a && AProtect(a))) && (!nicktemp->user || !IsAnAdmin(nicktemp->user))))
					{
						csmode(ch, "-b %s", param);
						break;
					}
					break;
				case 'l':
					if(what && is_num(parv[++i])) limit = parv[i];
					break;
				case 'k':
					key = parv[++i];
					break;
			}
			modes++;
		}
		if(ch) parse_cmode(ch, parv[2], limit, key);
	}
	else if((nick = getnickbynick(parv[1]))) parse_umode(nick, parv[2]);
	return;
}

/*
 * m_part
 *
 * parv[0] = nick
 * parv[1] = chans
 * parv[2] = raison
 */
void m_part(int parc, char **parv)
{
	char *p;
	aNick *nick = NULL;

	if(parc < 2) return;
#ifdef N2K
	nick = num2nickinfo(parv[0]);
#else
	nick = getnickbynick(parv[0]);
#endif
	for(p = strtok(parv[1], ",");p;p = strtok(NULL, ",")) del_join(nick, p);
	return;
}

/*
 * m_kick
 *
 * parv[0] = kickeur fou
 * parv[1] = chan
 * parv[2] = num victime
 * parv[3] = raison
 */
void m_kick(int parc, char **parv)
{
	aNick *k, *v;
	aChan *ch;
	aJoin *j = NULL;
	anAccess *a1 = NULL, *a2 = NULL;

	if(parc < 3) return;
	if(!(ch=getchaninfo(parv[1]))) return;
	if(!strcmp(cs.num,parv[2])) /*on sait jamais*/
	{
		csjoin(ch);
		csmode(ch, "+o %s", cs.num);
		return;
	}

	if(!(v=num2nickinfo(parv[2]))) return;
	del_join(v, parv[1]);

	if(!v->user) return;/*v = pas auth*/
#ifdef N2K
	k = num2nickinfo(parv[0]);
#else
	k = getnickbynick(parv[0]);
#endif
	if(!k) return;
	if(IsAnAdmin(v->user))/*cas v = admin*/
	{
		if(k->user && k->user->level > v->user->level) return; /*k = admin superieur*/
		csmode(ch, "-o %s", k->numeric);/*k non auth, user ou admin inferieur*/
		if((j = getjoininfo(k, ch->chan))) j->is_op = 0;
		return;
	}
	else
	{
		if(k->user && IsAnAdmin(k->user)) return;/*k=admin et v non*/
		a1=GetAccessIbyUserI(v->user,parv[1]);
		if(!a1 || !AProtect(a1) || ASuspend(a1)) return;/*v a pas dacces ou non protect*/
		if(k->user)
		{
			a2=GetAccessIbyUserI(k->user,parv[1]);
			if(a2 && a2->level >= a1->level && !ASuspend(a2)) return; /*meme niveau*/
			csmode(a1->c, "-o %s", k->numeric);
			if((j = getjoininfo(k, ch->chan))) j->is_op = 0;
			return;
		}
		csmode(a1->c, "-o %s", k->numeric);
		if((j = getjoininfo(k, ch->chan))) j->is_op = 0;
	}
	return;
}

/*
 * m_join
 *
 * parv[0] = nick
 * parv[1] = chans
 */
void m_join(int parc, char **parv)
{
	char *p;
	aNick *nick;

	if(parc < 2) return;
#ifdef N2K
	nick = num2nickinfo(parv[0]);
#else
	nick = getnickbynick(parv[0]);
#endif
	if(strchr(nick->nick, '@')) return;
	if(parv[1][0]=='0') /* partall !!*/
	{
		aJoin *join, *jt;
		for(join=nick->joinhead;join;join=jt)
		{
			jt=join->next;
			del_join(nick, join->channel);
		}
		return;
	}
	for(p = strtok(parv[1], ",");p;p=strtok(NULL, ",")) add_join(nick, p, 0, 0, time(NULL), 0);
	return;
}

/*
 * m_create
 *
 * parv[0] = num
 * parv[1] = chan
 * parv[2] = timestamp
 */
void m_create(int parc, char **parv)
{
	char *p;
	aNick *nick=NULL;

	if(parc < 3) return;
	nick = num2nickinfo(parv[0]);
	for(p = strtok(parv[1], ",");p;p = strtok(NULL, ",")) add_join(nick, p, 1, 0, atol(parv[2]), 0);
	return;
}

#ifndef N2K
/*
 * m_setident
 *
 * parv[0] = num
 * parv[1] = host
 */
void m_setident(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc < 2) return;
	strncpy(who->ident, parv[1], USERLEN);
	who->ident[USERLEN] = 0;
	return;
}

/*
 * m_sethost
 *
 * parv[0] = num
 * parv[1] = host
 */
void m_sethost(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc < 2) return;

	if(parv[1][0]=='\0' || strchr(parv[1], ':') || !strchr(parv[1], '.')) return;

#ifdef HAVE_CRYPTHOST
	strncpy(who->crypt, parv[1], HOSTLEN);
	who->crypt[HOSTLEN] = 0;
#else
	strncpy(who->host, parv[1], HOSTLEN);
	who->host[HOSTLEN] = 0;
#endif

	return;
}
#endif

/*
 *m_nick
 *
 * parv[1] = nick
 * parv[2] = hop
 * parv[3] = timestamp
 * parv[4] = username
 * parv[5] = host
 *  ==[ user a un/des mode(s) | user a pas de mode ]==
 * parv[6] = modes | base64
 * parv[7] = base64 | num
 * parv[8] = num | name
 * parv[9] = name |
 */
void m_nick(int parc, char **parv)
{
	char *ident, *nick, *host, *num = NULL, *lastnick = NULL, *ac = NULL
#ifdef HAVE_CRYPTHOST
, key[32]
#endif
;
	aNick *who = NULL;
	anUser *user;
	struct Link *serv;
	struct badnicks *badnick;
	int i = 6;

	if(parc > 7)
	{  /*Nouveau nick*/
		ident = parv[4];
		host = parv[5];
		if(*host=='+') return;
		nick = parv[1];

		if(parv[6][0] == '+')
		{
			i = 7;
			if(strchr(parv[6], 'r')) ac = parv[i++];
		}

		who = add_nickinfo(nick, ident, host, parv[i], parv[i+1], (serv = num2servinfo(parv[i+1])), parv[i+2], atol(parv[3]));

		if(parv[6][0] == '+') parse_umode(who, parv[6]);

#ifdef HAVE_CRYPTHOST
		if(strchr(host, '.')) /* la clef est la premire partie de l'host*/
		{
			if(!is_ip(host))
			{
				strncpy(key, host, 32);
				strtok(key, ".");
			}
			else strncpy(key, strchr(host, '.'), 32);
		}
		strncpy(who->crypt, hostprot(key, host), HOSTLEN); /* cryptage de l'host en md2 ou md5 suivant certains critres*/
		who->crypt[HOSTLEN] = 0;
#endif
		num = who->numeric;

		if(ac)
		{
			if(!(user=getuserinfo(ac)))
			{
				cswallops("ACCOUNT inconnu (%s) pour %s depuis %s", ac, who->nick, serv->serv);
				cs_account(who, NULL);
				return;
			}
			if(user->n)
			{
				cswallops("ACCOUNT DESYNCH (%s) pour %s depuis %s", ac, who->nick, serv->serv);
				cs_account(who, NULL);
				return;
			}
			who->user = user;
			who->user->n = who;
			who->user->lastseend = time(NULL);
			csntc(who, "Vous avez t \2ridentifi\2");
#ifdef NICKSERV
			if(UProtect(user)) del_killinfo(who->nick, who->numeric); /* arret du kill */
#endif
		}

#ifdef USE_WELCOMESERV
		if((ConfFlag & CF_WELCOME) && !burst) /* Envoi du welcome si il est actif et non nul*/
			choose_welcome(who->numeric);
#endif

	}
	else if(parc <= 3)
	{  /* Changement de pseudo */
		nick = parv[1];
		num = parv[0];
		who = num2nickinfo(num);

		del_killinfo(who->nick, num);
		lastnick = who->nick;
		strncpy(who->nick, nick, NICKLEN);
		who->nick[NICKLEN] = 0;
	}
	else return;

	if(!who) who = num2nickinfo(num);/*on refait pas la loop*/

	if((badnick = getbadnick(nick)) && !(badnick->flag & CF_NOREG))
	{
		csntc(who, "\2Avertissement\2 : Vous venez de prendre un pseudo illgal");
		if(!(ConfFlag & CF_CHANGENICK))
			csntc(who, "Veuillez changer de pseudo sinon vous serez dconnect dans %d secondes.", kill_interval);
		add_killinfo(who->nick, who->numeric, time(NULL), 1, (ConfFlag & CF_CHANGENICK) ? 0 : 1);
		return;/*on check meme pas le reste..*/
	}
	if(who->user && !strcasecmp(who->user->nick, nick)) return;/*le mec est auth sur ce nick > exit*/

#ifdef USE_NICKSERV
	if(!(user = getuserinfo(nick))) return;/*le pseudo n'est pas reg, on evite leisregnick > exit*/
	csntc(who, "\2Avertissement\2 : Le pseudo %s est enregistr, si c'est le votre, identifiez vous via: /%s %s %s <votre pass>", nick, cs.nick, RealCmd("login"), nick);
	if(UProtect(user))
	{
		if(UKill(user)) csntc(who, "Veuillez changer de pseudo sinon vous serez dconnect dans %d secondes.", kill_interval);
		else csntc(who, "Veuillez changer de pseudo sinon votre pseudo sera automatiquement chang dans %d secondes.", kill_interval);
		add_killinfo(who->nick, who->numeric, time(NULL), 2, UKill(user));
	}
	else csntc(who, "Le propritaire peut le rcuprer en vous dconnectant.");
#endif
	return;
}

/*
 *
 *
 * parv[0] = num
 * parv[1] = chan
 * parv[2] = timestamp
 * parv[3] = modes / users
 * parv[4] = users / bans
 * parv[5] = bans
 */
void m_burst(int parc, char **parv)
{
	aNick *nick = NULL;
	aLink *j = NULL;
	aChan *c;
	char *modes = NULL, *chan = parv[1], *timestamp = parv[2], *users = NULL, *key = NULL, *limit = NULL;
	char *flags = NULL, *num = NULL, *pv = NULL/*, *bans = NULL*/;
	int op = 0, voiced = 0, i = 4;
	long il = atol(timestamp);

	if(parc > 2 && parv[3][0] == '%') return;

	if(parc > 4 && parv[3][0] == '+')
	{  /* channel has mode(s) */
		modes = parv[3];
		if(strchr(modes, 'l')) limit = parv[i++];
		if(strchr(modes, 'k')) key = parv[i++];
		users = parv[i];
		/*if(parc > i) bans = parv[++i];*/
	}
	else
	{  /* channel has no mode */
		modes = "";
		users = parv[3];
		/*if(parc > 4) bans = parv[4];*/
	}

	if(!burst)
	{
		if((c=getchaninfo(chan)))
		{
			if (!c->chanstamp || il < c->chanstamp)
			{ /* les users du chan vont se faire deop et les modes rinitialiss comme le veut le serveur qui envoi le burst*/
				parse_cmode(c, modes, limit, key);
				putserv("%s %s %s +o %s\n", bot.servnum, TOKEN_MODE, c->chan, cs.num);
				c->chanstamp = il;
				for(j=c->members;j;j=j->next)
				{
					/* le pb ici c'est qu'on ne peut dterminer un user qui s'est fait deop UNIQUEMENT sur les
					chans regs, donc si un mec tait op sur un chan non reg, que le timestamp le fasse se deop,
					et qu'il veut reg le chan, il pourra sans etre op (bien qu'il l'tait tout de mme avant)
					Toute faon  part a on s'en fou vu que les chans sont pas reg on en a rien  battre */
					j->value.j->is_op = j->value.j->is_voice = 0;
				}
			}
		}
	}
	else
	{
		if(!strcasecmp(bot.pchan, chan))
		{
#ifdef USE_WELCOMESERV
			putserv("%s %s %s %ld +mints %s:ov,%s\n", bot.servnum, TOKEN_BURST, bot.pchan, il, ws.num, cs.num);
#else
			putserv("%s %s %s %ld +mints %s:ov\n", bot.servnum, TOKEN_BURST, bot.pchan, il, cs.num);
#endif
			burst = -1 * burst; /* pour remplacer le 'gotcs' local, sans rajouter de global var*/
		}
		else if((c=getchaninfo(chan)) && !IsSuspend(c) && !CJoined(c))
		{
			putserv("%s %s %s %ld %s %s:o\n", bot.servnum, TOKEN_BURST, c->chan, il, c->defmodes, cs.num);
			parse_cmode(c, modes, limit, key);
			if(CLockTopic(c))
			{
				if(c->topic[0] != '\0') cstopic(c, "%s", c->topic);
				else if(c->deftopic[0] != '\0') cstopic(c, "%s", c->deftopic);
			}
			SetCJoined(c);
			c->chanstamp = il;
		}
	}

	pv = users;
	while((num=strtok(pv, ",")))
	{
		if((flags = strchr(num, ':')))
		{
			op = voiced = 0;
			*flags = '\0';
			while (*(++flags))
			if (*flags == 'o') op = 1;
			else if (*flags == 'v') voiced = 1;
			else break;
		}
		if((nick=num2nickinfo(num))) add_join(nick, chan, op, voiced, il, 1);
		pv=NULL;

	}
 /* if(bans)
  {
  	  bans += 2;
	  pv=bans;
	  while((mask=strtok(pv, " ")))
	  {
	        add_ban(chan, mask, "Aucune", "MODE", time(NULL), 0, 0);
	  	pv=NULL;
	  }
  }*/
	return;
}

/*
 * m_quit
 *
 * parv[0] = nick
 * parv[1] = msg
 */
void m_quit(int parc, char **parv)
{
	aNick *nick;

#ifdef N2K
	nick = num2nickinfo(parv[0]);
#else
	nick = getnickbynick(parv[0]);
#endif
	if(!nick) return; /* !!!*/
	if(nick->user)
	{
#ifdef HAVE_TRACK
		struct track *track = NULL, *tt;
		extern struct track *trackhead;

		for(track=trackhead;track;track=tt)
		{
			tt = track->next;
			if(nick->user == track->tracked)
			{
				csntc(track->tracker, "[\2Track\2] %s (User: %s) vient de quitter.", nick->nick, track->tracked->nick);
				break;
			}
			if(nick == track->tracker) del_track(track->tracked);
		}
#endif
	}
	del_killinfo(nick->nick, nick->numeric);
	del_nickinfo(nick);
	return;
}

/*
* m_kill
* Nettoyage des vars
*/

void m_kill(int parc, char **parv)
{
	const char *num = parv[1];
	aNick *nick=NULL;

	if(!(nick = num2nickinfo(num))) return;
	if(nick->user)
	{
#ifdef HAVE_TRACK
		struct track *track = NULL, *tt;
		extern struct track *trackhead;

		for(track=trackhead;track;track=tt)
		{
			tt = track->next;
			if(nick->user == track->tracked)
			{
				csntc(track->tracker, "[\2Track\2] %s (User: %s) vient d'tre Killer.", nick->nick, track->tracked->nick);
				break;
			}
			if(nick == track->tracker) del_track(track->tracked);
		}
#endif
	}
	del_killinfo(nick->nick, num);
	del_nickinfo(nick);

	return;
}

/*
 * m_away
 *
 * parv[0] = who
 * parv[1-parc] = raison
 */
void m_away(int parc, char **parv)
{
	aNick *nick;

#ifdef N2K
    if(!(nick = num2nickinfo(parv[0]))) return;
#else
    if(!(nick = getnickbynick(parv[0]))) return;
#endif
	if(parc > 1 && strlen(parv[1]) > 0) nick->flag |= N_AWAY;
	else
	{
		nick->flag &= ~N_AWAY;
#ifdef USE_MEMOSERV
		if(nick->user) show_notes(nick);
#endif
	}
	return;
}

/*
 * m_whois
 *
 * parv[0] = who
 * parv[1] = ma num
 * parv[2] = pseudo du robot
 */
void m_whois(int parc, char **parv)
{
	char insulte[100];
	int i;
	aNick *tmp;

	if(!strcasecmp(parv[2], cs.nick))
		putserv("%s 311 %s %s %s %s * :%s\n", bot.servnum, parv[0], cs.nick, cs.ident, cs.host, cs.name);
#ifdef USE_WELCOMESERV
	else if(!strcasecmp(parv[2], ws.nick))
		putserv("%s 311 %s %s %s %s * :%s\n", bot.servnum, parv[0], ws.nick, ws.ident, ws.host, ws.name);
#endif
	else
	{
		if(!(tmp = getnickbynick(parv[2]))) return;

		i = rand()%10;
		if(!i) strcpy(insulte, "a une face de cul de babouin !");
		else if(i == 1) strcpy(insulte, "est une lamouze !");
		else if(i == 2) strcpy(insulte, "est transsexuel !");
		else if(i == 3) strcpy(insulte, "a un QI ngatif !");
		else if(i == 4) strcpy(insulte, "a la diahrre !");
		else strcpy(insulte, "est completement dbile !");
#ifdef HAVE_CRYPTHOST
		putserv("%s 311 %s %s %s %s * :%s\n", bot.servnum, parv[0], parv[2], tmp->ident, tmp->crypt, tmp->name);
#else
		putserv("%s 311 %s %s %s %s * :%s\n", bot.servnum, parv[0], parv[2], tmp->ident, tmp->host, tmp->name);
#endif
		putserv("%s 312 %s %s %s :%s\n", bot.servnum, parv[0], parv[2], tmp->serveur?tmp->serveur->serv:bot.server, tmp->name);
		putserv("%s 390 %s %s :%s\n", bot.servnum, parv[0], parv[2], insulte);
		if((tmp->flag & N_OPER)) putserv("%s 313 %s %s :est un IRC Oprateur tyrannique et pervers\n", bot.servnum, parv[0], parv[2]);
		putserv("%s 317 %s %s 99999999 99999999 :Secondes d'idle, secondes d'uptime\n", bot.servnum, parv[0], parv[2]);
		putserv("%s 318 %s %s :Fin de la commande /WHOIS\n", bot.servnum, parv[0], parv[2]);
		return;
	}
	putserv("%s 312 %s %s %s :%s\n", bot.servnum, parv[0], parv[2], bot.server, bot.name);
	putserv("%s 390 %s %s :est un Robot\n", bot.servnum, parv[0], parv[2]);
	putserv("%s 313 %s %s :est un IRC Oprateur - Service\n", bot.servnum, parv[0], parv[2]);
	putserv("%s 317 %s %s %ld %ld :Secondes d'idle, secondes d'uptime\n", bot.servnum, parv[0], parv[2], time(NULL), tuptime);
	putserv("%s 318 %s %s :Fin de la commande /WHOIS\n", bot.servnum, parv[0], parv[2]);
	return;
}

void m_topic(int parc, char **parv)
{
	aChan *chan = getchaninfo(parv[1]);

	if(!chan) return;
	if(CLockTopic(chan))
	{
		if(chan->topic[0] != '\0') cstopic(chan, "%s", chan->topic);
		else cstopic(chan, "%s", chan->deftopic);
	}
#ifdef N2K
	else strncpy(chan->topic, parv[3], TOPICLEN);
#else
	else strncpy(chan->topic, parv[2], TOPICLEN);
#endif
	chan->topic[TOPICLEN] = 0;
	return;
}

/*
 * m_eob
 *
 * parv[0] = who
 * parv[1] = me :)
 */
void m_eob(int parc, char **parv)
{
	struct Link *serv = NULL;

	if (!(serv=num2servinfo(parv[0])))
	{
		putlog(LOG_ERREURS, "ATTENTION: Prfixe serveur reu incorrect\n");
		return ;
	}
	if(!(serv->flag & ST_BURST))
	{
		putlog(LOG_ERREURS, "ATTENTION: Erreur interne  m_eob()\n");
		return ;
	}
	serv->flag &= ~ST_BURST;
	serv->flag |= ST_ONLINE;
	if(serv == mainhub)
	{
		aChan *chan;
		struct timeval tv;
		float tt;

#ifdef USE_WELCOMESERV
		if(burst>0) putserv("%s %s %s %ld +mints %s:ov,%s\n", bot.servnum, TOKEN_BURST, bot.pchan, time(NULL), cs.num, ws.num);
#else
		if(burst>0) putserv("%s %s %s %ld +mints %s:ov\n", bot.servnum, TOKEN_BURST, bot.pchan, time(NULL), cs.num);
#endif
		else if(burst < 0) burst = -1*burst;

		for(chan = chanhead;chan;chan=chan->next)
		{
			char tmp[30], *p = NULL, *key = NULL, *limit = NULL;
			if(!CJoined(chan) && !IsSuspend(chan))
			{
				putserv("%s %s %s %ld %s %s:o\n", bot.servnum, TOKEN_BURST, chan->chan, time(NULL), chan->defmodes, cs.num);
				if(chan->defmodes[0] != '\0')
				{
					strcpy(tmp, chan->defmodes);
					p = strtok(tmp, " ");
					while(*p != '\0' && *p != ' ')
					{
						if(*p == 'k') key = strtok(NULL, " ");
						if(*p == 'l') limit = strtok(NULL, " ");
						p++;
					}
					parse_cmode(chan, tmp, limit, key);
				}
				if(chan->topic[0] != '\0') cstopic(chan, "%s", chan->topic);
				else if(chan->deftopic[0] != '\0') cstopic(chan, "%s", chan->deftopic);
				SetCJoined(chan);
				chan->chanstamp = time(NULL);
			}
		}

		putserv("%s %s\n", bot.servnum, TOKEN_EOB);
		putserv("%s %s\n", bot.servnum, TOKEN_EOBACK);

		if(dataQ)
		{
			gettimeofday(&tv, NULL);/*un peu de stats sur le burst pour *fun* */
			tt = (tv.tv_usec - burst)/1000000.0;
			if(tt<0) tt= -1*tt;
			cswallops("Synchronisation de %d bytes en %12.6f s (Taux %12.6f Ko/s)", dataQ, tt,(float) (dataQ / 1024.0) / tt);
		}
		burst=0;
	}
	alarm(ALARMINTERVAL);
	return;
}

static void ClearNicksbyServ(struct Link *link)
{
	aNick *nick, *nt;

	for(nick = nickhead;nick;nick=nt)
	{
		nt = nick->next;
		if(nick->serveur == link) del_nickinfo(nick);
	}
}

void m_server(int parc, char **parv)
{
	struct Link *link, *last;

	if(!(link = calloc(1, sizeof(struct Link))))
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un Link %s !\n", parv[1]);
		return;
	}

	strncpy(link->serv,parv[1],150);
#ifdef N2K
	strncpy(link->num,parv[6],2);
#else
	strncpy(link->num,parv[6],1);
#endif
	link->flag = 0;
	link->next = NULL;

	if(atoi(parv[2]) == 1)/*1erhub*/
	{
		link->hub = NULL;
		link->last = NULL;
		mainhub = link;
		registered = 1;
	}
	else
	{
		last=num2servinfo(parv[0]);
		link->hub = last; /* ptr vers son Hub */
		last->flag |= ISHUB;
		for(last=mainhub;last->next;last=last->next); /* get last ptr*/
		last->next = link; /* chainage positif */
		link->last = last; /* chainage ngatif */
	}

	if (parv[5][0] == 'J') link->flag |= ST_BURST;
	else link->flag |= ST_ONLINE;
	return;
}

static void NullHub(struct Link *hub)
{
	struct Link *l;
	for(l=hub;l;l=l->next) if(l->hub == hub) l->hub = NULL;
}

void m_squit(int parc, char **parv)
{
	struct Link *link = NULL, *last = NULL, *lt = NULL;

	/*nettoyage des serveurs en "dessous" (dont il est Hub) */

	for(link=mainhub;link && strcasecmp(parv[1], link->serv);link=link->next);
	if(link)
	{
		if(link == mainhub) /* squit de l'uplink, tentative de reconnexion.. */
		{
			for(;link;link=lt)
			{
				lt=link->next;
				free(link);
			}
			mainhub = NULL;
			return;
		}

		if(link->next)
		{
			link->next->last = link->last;
			link->last->next = link->next;
		}
		else link->last->next = NULL;

		if(link->flag & ISHUB)
		{
			NullHub(link); /* s'il est hub ya forcement un leaf (next) apres!? */
			for(last=link->next;last;last=lt)
			{
				lt=last->next;
				if(!last->hub)
				{
					ClearNicksbyServ(last);
					if(last->flag & ISHUB) NullHub(last);/*  on met les ptr vers ce hub (leaf du premier) a NULL*/
					last->next->last = last->last;/*		 ses leafs seront del lors de la suite de la boucle*/
					last->last->next = last->next;
					free(last);
					last = NULL;
				}
			}
		}
		ClearNicksbyServ(link);
		free(link);
		link = NULL;
	}

	return;
}

void m_pass(int parc, char **parv)
{
	struct timeval tv;
	time_t current = time(NULL);

	gettimeofday(&tv, NULL);
	burst = tv.tv_usec;

	putlog(LOG_PARSES, "Connect au serveur %s\n", bot.ip);
	registered = 0;
	if(!tuptime) tuptime = current;
	if(!lastresetlogs) lastresetlogs = current;
	if(!lastresetparse) lastresetparse = current;
	return;
}

void m_ping(int parc, char **parv)
{
#ifdef N2K
	struct timeval tv;
	char *ptr = parv[1], *theirttm = strchr(parv[1], '.');

	if(*theirttm == '.') theirttm++;
	if(*ptr == '!') ptr++;
	gettimeofday(&tv, NULL);
/* AsLL */
	putserv("%s %s %s %s %s %d %ld.%d\n", bot.servnum, TOKEN_PONG, bot.servnum, parv[0], ptr, (int) (tv.tv_usec - atoi(theirttm)), time(NULL), (int) tv.tv_usec);
#else
    putserv("%s %s %s\n", bot.servnum, TOKEN_PONG, parv[1]);
#endif
    return;
}

void exec_cmd(unsigned int i, unsigned int securise, char *ni, int parc, char **parv)
{
	char tmp3[400], buff[512], tmp[(2 * (NICKLEN + 1))];
	aChan *chaninfo = NULL;
	anAccess *access = NULL;
	aNick *nick = NULL;
#ifdef HAVE_TRACK
	struct track *track = NULL;
#endif

#ifdef N2K
	if(!(nick=num2nickinfo(ni))) return; /* !!!!*/
#else
	if(!(nick=getnickbynick(ni))) return; /* !!!!*/
#endif

	if(isignore(nick) || checkflood(nick)) return;

	parv2msgn(parc, parv, 1, tmp3, 400);
#ifdef HAVE_SECURE
	if(cs_cmds[i].secure && !securise && (strcasecmp(cs_cmds[i].realcmd, "memos") || !parc || !strcasecmp(parv[1], "SEND")))
	{
		csntc(nick, "La commande \002%s\2 demande des informations prives. Veuillez utiliser \2/%s %s %s\2.", cs_cmds[i].cmd, cs.nick, cs_cmds[i].cmd, tmp3);
		return;
	}
#endif

	cs_cmds[i].used++;
	if((cs_cmds[i].flag & CMD_DISABLE))
	{
		csntc(nick, "\2La commande %s est dsactive.", cs_cmds[i].cmd);
		return;
	}
	if(parc < cs_cmds[i].args || (!parc && ChanCmd(i)))
	{
		syntax_cmd(nick, i);
		return;
	}

	if(ChanCmd(i))
	{
		if(!(chaninfo = getchaninfo(parv[1])))
		{
			csntc(nick, NOSUCHAN, parv[1]);
			return;
		}
		if(IsSuspend(chaninfo))
		{
			csntc(nick, "%s est suspendu : %s - %s", parv[1], chaninfo->suspend->raison, chaninfo->suspend->from);
			return;
		}
		if(nick->user) access = GetAccessIbyUserI(nick->user, parv[1]);
	}
	if(cs_cmds[i].level)
	{
		if(!nick->user || !nick->user->n)
		{
			csntc(nick, "%s : Vous n'tes pas logu ou vous n'avez pas encore enregistr d'Username", pasdeperm);
			return;
		}
		if(AdmCmd(i) && nick->user->level < cs_cmds[i].level)
		{
			csntc(nick, "%s : Vous devez tre Administrateur pour utiliser cette commande", pasdeperm);
			return;
		}
		if(ChanCmd(i) && !IsAnAdmin(nick->user))
		{
			if(!access)
			{
				csntc(nick, "%s : Vous n'avez pas d'accs sur le salon %s", pasdeperm, parv[1]);
				return;
			}
			if(cs_cmds[i].level > access->level)
			{
				csntc(nick, "%s : Votre niveau sur %s est trop petit (Vous: \002%d\002, Commande: \002%d\002)", pasdeperm, parv[1], access->level, cs_cmds[i].level);
				return;
			}
			if(ASuspend(access))
			{
				csntc(nick, "%s : Votre accs est suspendu sur %s", pasdeperm, parv[1]);
				return;
			}
		}
	}

	snprintf(tmp, 2 * NICKLEN, "%s%s%s", nick->nick, nick->user ? "@" : "", nick->user ? nick->user->nick : "");
	if(cs_cmds[i].secure == 1 && parc) sprintf(buff, "\002%s\2 %s par %s", cs_cmds[i].cmd, parv[1], tmp);
	else if(cs_cmds[i].secure == 2 && parc) sprintf(buff, "\002%s\2 par %s", cs_cmds[i].cmd, tmp);
	else if(parc >= 2
		&& ((!strcasecmp(cs_cmds[i].realcmd, "USER") && !strcasecmp(parv[1], "NEWPASS"))
		|| (!strcasecmp(cs_cmds[i].realcmd, "SET") && !strcasecmp(parv[1], "PASS"))))
		sprintf(buff, "\002%s\2 %s par %s", cs_cmds[i].cmd, parv[1], tmp);
	else if(parc) sprintf(buff, "\002%s\2 %s par %s", cs_cmds[i].cmd, tmp3, tmp);
	else sprintf(buff, "\002%s\2 par %s", cs_cmds[i].cmd, tmp);

#ifdef HAVE_TRACK
	if(nick->user && (track = istrack(nick->user))) csntc(track->tracker, "[\2Track\2] %s", buff);
#endif
	putchan(buff);
	putlog(LOG_CMDS, "%s\n", buff);
	cs_cmds[i].func(nick, chaninfo, parc, parv);/*permet de passer la struct chaninfo (economie de loop)*/
	return;
}

/*
 * m_privmsg
 *
 * parv[0] = envoyeur
 * parv[1] = destinataire (pseudo du robot, set dans botnick)
 * parv[2] = message reu.
 *
 * pparv[0] = nom de la commande
 * pparv[pparc] = arguments de la commande
 * ATTENTION: pparc est le nb d'arg  partir de *parv[1]*
 */
void m_privmsg(int parc, char **parv)
{
	char *tmp, *cmd = NULL, *botnick = NULL;
	char *pparv[250], tmp2[500] = {0};
	aNick *ni=NULL;
	int i, pparc = 0, securise = 0;

	if(*parv[1] != '#')
	{  /* Message envoy  un robot*/

		pparv[1] = NULL;

		if(strchr(parv[1], '@'))
		{
			botnick = strtok(parv[1], "@");
			securise = 1;
		}
		else botnick = parv[1];

		if(parv[2][0]=='\0' || parv[2][0]==' ') return;

		tmp = strtok(parv[2], " ");
		if (tmp)
		{
			cmd = tmp;
			pparv[0] = cmd;
			tmp = strtok(NULL, " ");
		}
		while (tmp)
		{
			pparc++;
			pparv[pparc] = tmp;
			tmp = strtok(NULL, " ");
		}

#ifdef N2K
		if(premiere && strcasecmp("HELLO", cmd) && (ni = num2nickinfo(parv[0])))
#else
		if(premiere && strcasecmp("HELLO", cmd) && (ni = getnickbynick(parv[0])))
#endif
		{
			csntc(ni, "%s n'a pas encore t enregistr", cs.nick);
			return;
		}

		if (!strcmp(botnick, cs.num) || !strcasecmp(botnick, cs.nick))
		{
			if((i = GetCurCmdIndexbyName(cmd)) < 0)
			{
#ifdef N2K
				if(*cmd != '\001' && (ni = num2nickinfo(parv[0]))) csntc(ni, "La commande \002%s\002 n'existe pas.", cmd);
#else
				if(*cmd != '\001' && (ni = getnickbynick(parv[0]))) csntc(ni, "La commande \002%s\002 n'existe pas.", cmd);
#endif
			}
			else
			{
				exec_cmd(i, securise, parv[0], pparc, pparv);
				return;
			}

		}/*endif*/
		return;
	}
	else
	{  /* Message envoy sur un salon (pour chanserv)*/
		if(parv[2][0] == bot.cara && parv[2][1] != '\0')
		{
			parv[2]++;

			if(parv[2][0]=='\0' || parv[2] == NULL) return;

			tmp = strtok(parv[2], " ");
			if (tmp != NULL)
			{
				cmd = tmp;
				pparv[0] = cmd;
				tmp = strtok(NULL, " ");
			}

#ifdef N2K
			if(premiere && strcasecmp("HELLO", cmd) && (ni = num2nickinfo(parv[0])))
#else
			if(premiere && strcasecmp("HELLO", cmd) && (ni = getnickbynick(parv[0])))
#endif
			{
				csntc(ni, "%s n'a pas encore t enregistr", cs.nick);
				return;
			}

			if((i = GetCurCmdIndexbyName(cmd)) < 0) return;

			do
			{
				if(ChanCmd(i) && pparc == 0 && (tmp == NULL || tmp[0] != '#'))
				{
					pparc++;
					pparv[pparc] = parv[1];
					strcat(tmp2, " ");
					strcat(tmp2, parv[1]);
				}
				else if (tmp != NULL)
				{
					pparc++;
					pparv[pparc] = tmp;
					strcat(tmp2, " ");
					strcat(tmp2, tmp);
					tmp = strtok(NULL, " ");
				}
			}
			while (tmp);
			exec_cmd(i, securise, parv[0], pparc, pparv);
			return;
		}
	}
	return;
}

void parse_this(char *msg)
{
	int parc = 0, i = 0;
	char *ptr, *ptr2, *cmd;
	char *parv[MAXPARA + 2 ]; /* leave room for prefix and end-of-params NULL */
#ifdef DEBUG
	char logg[512];
	strncpy(logg,msg,512);
#endif

	ptr = strchr(msg, ' ');
	if (!ptr) return;


#ifndef  N2K
	if (msg[0] == ':')
	{
		/* long prefix */
		parv[0] = &msg[1];
		*ptr = '\0';
		ptr++;
	}
	else if (ptr-msg && strncmp(msg, "SERVER", 6) && strncmp(msg, "PASS",4) && (registered))
#else
	if (ptr-msg && strncmp(msg, "SERVER", 6) && strncmp(msg, "PASS",4) && (registered))
#endif
	{
		/* numeric prefix */
		parv[0] = msg;
		*ptr = '\0';
		ptr++;
	} else
	{
		/* no prefix */
		parv[0] = bot.server;
		ptr=msg;
	}

	ptr2 = strchr(ptr, ' ');
	cmd = ptr;
	if (ptr2)
	{
		*ptr2 ='\0';
		ptr2++;
		ptr = ptr2;
		for (parc=1; (parc <= MAXPARA) && ptr; parc++)
		{
			ptr2 = strchr(ptr, ' ');
			if (!ptr2)
			{
				if (*ptr == ':') ptr++;
				parv[parc]=ptr;
				break;
			} else
			{
				if (*ptr == ':')
				{
					ptr++;
					parv[parc] = ptr;
					break;
				}
				*ptr2='\0';
				ptr2++;
				if (*ptr == ':')
				{
					ptr++;
					parv[parc] = ptr;
					break;
				}
				parv[parc] = ptr;
				ptr = ptr2;
			}
		}/*endfor*/

	} else parc=0;
	parc++;
	parv[parc] = NULL;

#ifdef DEBUG /*etical loging*/
	if(!(parc > 1 && ((!strcasecmp(cmd,TOKEN_PRIVMSG) && parv[1][0] == '#') ||
         !strcasecmp(cmd, TOKEN_PING) || !strcasecmp(cmd, TOKEN_PONG))))
	putlog(LOG_PARSES, "R - %s\n", logg);
#endif

	for (; msgtab[i].cmd != NULL; i++)
	{
		if (!strcasecmp(msgtab[i].cmd,cmd))
		{
			msgtab[i].func(parc, parv);
			return;/*on sait jamais */
		}
	}
}

int init_bot(int ip, int port)
{
	int sock, r;
	time_t LTS = time(NULL);
	struct sockaddr_in fsocket;
	aNick *nick;

	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (sock < 0) return(-1);

	fsocket.sin_family = AF_INET;
	fsocket.sin_addr.s_addr = ip;
	fsocket.sin_port = htons(port);
	r=connect(sock, (struct sockaddr *) &fsocket, sizeof(fsocket));

	if (r==(-1))
	{
		close(sock);
		return(-1);
	}

	bot.sock = sock;
	putserv("PASS %s\n", bot.pass);
#ifdef N2K
	putserv("SERVER %s 1 %u %u J10 %s]]] :%s\n", bot.server, STS, LTS, bot.servnum, bot.name);
#else
	putserv("SERVER %s 1 %u %u J10 %s]] :%s\n", bot.server, STS, LTS, bot.servnum, bot.name);
#endif

	putserv("%s %s %s 1 %u %s %s %s B]AAAB %s :%s\n", bot.servnum, TOKEN_NICK, cs.nick, LTS, cs.ident, cs.host, cs.mode, cs.num, cs.name);

	nick = add_nickinfo(cs.nick, cs.ident, cs.host, "B]AAAB", cs.num, NULL, cs.name, LTS);
	parse_umode(nick, cs.mode);
#ifdef HAVE_CRYPTHOST
	strcpy(nick->crypt, cs.host);
#endif

#ifdef USE_WELCOMESERV
	putserv("%s %s %s 1 %u %s %s +kpd B]AAAB %s :%s\n", bot.servnum, TOKEN_NICK, ws.nick, LTS, ws.ident, ws.host, ws.num, ws.name);
	nick = add_nickinfo(ws.nick, ws.ident, ws.host, "B]AAAB", ws.num, NULL, ws.name, LTS);
	nick->flag |= (N_SERVICE | N_DEAF | N_GOD);
#ifdef HAVE_CRYPTHOST
	strcpy(nick->crypt, ws.host);
#endif
#endif

  return sock;
}

#ifdef TESTBOT
int ircsock;
extern fd_set socks, master;
extern int highsock, bind_sock;
extern void w2c_setnonblocking(int sock);
extern struct w2c_clientinfo *w2c_clihead;
extern int bind_sock;
extern int CONtotal;
extern int WEBtrafficUP;
extern int WEBtrafficDL;
#endif

void run_bot(int sock)
{
#ifndef TESTBOT
	int r, t = 0;
	time_t next_ping, waitping;
	fd_set rfds;
	char buf[512], c;
#else
	char tbuf[512];
	int t = 0;
#endif

	if (sock < 0 ) return;

#ifdef TESTBOT

	ircsock = sock;
	w2c_setnonblocking(ircsock);
	if(sock > highsock) highsock = sock;
	FD_SET(sock, &master);

#else

	FD_ZERO(&rfds);
	FD_SET(sock, &rfds);
	next_ping = time(NULL) + PING_FREQ;
	waitping = 0;

#endif

	while (!want_to_quit)
	{
#ifdef TESTBOT

		if(!ircsock) break;
		if(w2c_checksocks())
		{
			int i = 0, r;

			for(;i <= highsock;i++)
			{
				if (!FD_ISSET(i, &socks)) continue;
				if(i == bind_sock)
				{
					int newfd;
					unsigned int addrlen;
					char ip[16] = {0};
					struct sockaddr_in remoteaddr;
					struct w2c_clientinfo *cli;

					addrlen = sizeof(remoteaddr);
					if ((newfd = accept(bind_sock, (struct sockaddr *)&remoteaddr, &addrlen)) == -1) continue;
					else
					{
						CONtotal++;
						FD_SET(newfd, &master);
						if (newfd > highsock) highsock = newfd;
						strcpy(ip, inet_ntoa(remoteaddr.sin_addr));
						cli = w2c_addclient(newfd, ip);
						w2c_sendrpl(cli, RPL_000); /*Hello World!*/
					}
				}
				else if(i == ircsock)
				{
					char buf[512] = {0}, *ptr = buf;

					r = recv(i, buf, 512, 0);
					if (r < 0)
					{
						ircsock = 0;
						close(i);
						FD_CLR(i, &socks);
					}
					buf[r] = '\0';
					while(*ptr)
					{
						/* lecture du buffer de recv(), puis dcoupage selon les \r\n et,
						enfin on garde en mmoire dans tbuf le dbut d'un paquet IRC s'il
						est incomplet. au select suivant les bytes lui appartenant lui seront
						ajoute grace  t, index permanent.       packet1\r\npacket2\r\n  */
						tbuf[t++] = *ptr;
						if(*ptr == '\n')
						{
							dataQ += t;/*comptage des bytes lues*/
							if(tbuf[t-2] == '\r') t -= 2;
							else t--;
							tbuf[t] = '\0';
							parse_this(tbuf);
							t = 0;
						}
						ptr++;
					}
				}
				else
				{
					char buf[1024] = {0};
					struct w2c_clientinfo *cli;

					if ((r = recv(i, buf, 1024, 0)) <= 0) w2c_delclient((cli = w2c_findclient(i)));
					else
					{
						WEBtrafficDL += r;
						cli = w2c_findclient(i);
						/* Faire une boucle de strtok comme pour le parse ircsock */
						w2c_parsemsg(cli, buf);
					}
				}

			}
			FD_ZERO(&socks);
		}
#else

		if (!FD_ISSET(sock, &rfds))
		{
			if (next_ping <= time(NULL))
			{
				if(!waitping)
				{
					putserv("%s %s :%s\n", bot.servnum, TOKEN_PING, bot.server);
					next_ping = time(NULL) + PING_FREQ;
					waitping = 1;
				}
				else
				{
					printf("Pas de rponse pour %s\n", bot.ip);
					close(sock);
					return;
				}
			}
			continue;
		}
		else
		{
			r = recv(sock, &c, 1, 0);
			if (r <= 0) break;
			next_ping = time(NULL) + PING_FREQ;
			waitping = 0;

			buf[t++] = c;
			if (c == '\n')
			{
				dataQ += t;/*comptage des bytes lues*/
				if(buf[t-2] == '\r') t -= 2;
				else t--;
				buf[t] = '\0';
				parse_this(buf);
				t = 0;
			}
		}

#endif

	}
	if (want_to_quit)
	{
		close(sock);
#ifdef TESTBOT
		w2c_closesocks();
#endif
	}
	return;
}
